%/*************************************************************************
%*  Copyright (c) 2010 Agilent Technologies Inc.  All rights reserved.
%*
%* You have a royalty-free right to use, modify, reproduce and distribute
%* the sample application files (and/or any modified version) in any way
%* you find useful, provided that you agree that Agilent Technologies has 
%* no warranty, obligations or liability for any sample application files.
%*
%* Agilent Technologies provides programming examples for illustration only,
%* This sample program assumes that you are familiar with the programming
%* language being demonstrated and the tools used to create and debug
%* procedures. Agilent Technologies support engineers can help explain the
%* functionality of Agilent Technologies software components and associated
%* commands, but they will not modify these samples to provide added
%* functionality or construct procedures to meet your specific needs.
%*************************************************************************/
% Agilent IVI-COM Driver Example Program
% 
% Creates a driver object, reads a few Identity interface
% properties, and checks the instrument error queue.
% May include additional instrument specific functionality.
% 
% Runs in simulation mode without an instrument.

function ML_BasicMeasurement()

    disp(blanks(1)');
    disp('  ML_BasicMeasurement')
    try
        % Create driver instance
        driver = instrument.driver.AgM9392;

        % Edit resource and options as needed.  Resource is ignored if option Simulate=true
        resourceDesc = 'PXI20::13::INSTR';
        % resourceDesc = 'TCPIP0::<host_name or IP addr>::INSTR'

        options = 'QueryInstrStatus=true, Simulate=true, DriverSetup= Model=, Trace=false';
        idquery = true;
        reset = true;

        % Initialize the driver.    
        % See driver help topic "Initializing the IVI-COM Driver" for information
        driver.Initialize(resourceDesc, idquery, reset, options);
        disp('Driver Initialized')

        % Print a few IIviDriver.Identity properties
        disp(['Identifier:        ', driver.Identity.Identifier])
        disp(['Revision:          ', driver.Identity.Revision])
        disp(['Vendor:            ', driver.Identity.Vendor])
        disp(['Description:       ', driver.Identity.Description])
        disp(['InstrumentModel:   ', driver.Identity.InstrumentModel])
        disp(['FirmwareRev:       ', driver.Identity.InstrumentFirmwareRevision])
        disp(['Serial #:          ', driver.DeviceSpecific.System.SerialNumber])
        simulate = driver.DriverOperation.Simulate;
        
        if simulate == true
            disp('Simulate:          True')
        else
            disp('Simulate:          False')
        end

        repeat = true;
        while(repeat)
            setupAcquisition(driver);
            if (true == binaryPrompt('Initiate', true))
                acquireMeasurement(driver,100);
            end
            repeat = binaryPrompt('Take another Measurement',true);
        end

        % Polling the instrument for queued errors is not necessary.  No errors are queued.
        % rather, each of the above driver calls would have thrown an exception if an error occurred.
        % Any thrown exceptions would be handled in the catch clause below:
    catch exception
        disp(getReport(exception))
        return;
    end

    if driver.Initialized
        try
            driver.Close();
            disp('Driver Closed');
        catch exception
            disp(getReport(exception));
            return;
        end
    end

    disp('Done')
    disp(blanks(1))
end

function reply = binaryPrompt(prompt,default)
% prompt the user for a binary (boolean) response 
    if default
        answer = input([prompt,' (enter "yes" or "no". default = "yes"): '],'s');
    else
        answer = input([prompt,' (enter "yes" or "no". default = "no"): '],'s');
    end
    if isempty(answer)
        reply = default;
    elseif(1 == strfind(lower(answer),'y'))
        reply = true;
    elseif(1 == strfind(lower(answer),'n'))
        reply = false;
    else
        disp 'Answer must be "yes" or "no".'
        reply = binaryPrompt(prompt,default);
    end
end

function reply = numberPrompt(prompt, min, max, mult, def)
    answer = input(sprintf('%s ( Enter a value between %f and %f default = %f):', prompt, min/mult, max/mult, def),'s');
    if isempty(answer)
        reply = def;
    else
        reply = sscanf(answer,'%f');
    end
    reply = reply * mult;
    if((reply<min)||(reply>max))
        printf('Entered value must be between %f and %f\n', min/mult, max/mult);
        reply = numberPrompt(prompt, min, max, mult, def);
    end
end

function setupAcquisition(driver)
	driver.DeviceSpecific.Acquisition.PreselectorEnabled = binaryPrompt(sprintf('Use Preselector(2.9-%fGHz)',  driver.DeviceSpecific.Acquisition.MaxFrequency/1000000000),false);
	driver.DeviceSpecific.Acquisition.Frequency = numberPrompt('Frequency (GHz)', driver.DeviceSpecific.Acquisition.MinFrequency, driver.DeviceSpecific.Acquisition.MaxFrequency, 1e9, 8.0);
    if((driver.DeviceSpecific.Acquisition.Frequency < 2.9e+9) && driver.DeviceSpecific.Acquisition.PreselectorEnabled)
		disp('     DISABLING preselector (frequency too low)\n');
		driver.DeviceSpecific.Acquisition.PreselectorEnabled = false;
    end
	driver.DeviceSpecific.Acquisition.Power = numberPrompt('Power (dBm)', driver.DeviceSpecific.Acquisition.MinPower, driver.DeviceSpecific.Acquisition.MaxPower, 1, -10);
	driver.DeviceSpecific.Acquisition.Bandwidth = numberPrompt('bandwidth (kHz)', 0, driver.DeviceSpecific.Acquisition.MaxBandwidth, 1000, 250);
	driver.DeviceSpecific.Acquisition.Time = numberPrompt('Acquisition Time(ms)', 0, driver.DeviceSpecific.Acquisition.MaxTime, 0.001, 10);
	driver.DeviceSpecific.Calibration.FieldCalibration.Enabled = binaryPrompt('Calibration',true);
end

function acquireMeasurement(driver,timeout)
    disp(blanks(1))
    disp('Initiate measurement...');
    driver.DeviceSpecific.Measurements.Initiate();
    driver.DeviceSpecific.Measurements.WaitForInitiateComplete(timeout);
    disp('Arming...');
    driver.DeviceSpecific.Measurements.Arm();

    driver.DeviceSpecific.Measurements.WaitForData(timeout);
 
    fprintf(1,'RMS Power = %f dBm.\n', driver.DeviceSpecific.Measurements.RmsPower);
    fprintf(1,'Peak Power = %f dBm.\n', driver.DeviceSpecific.Measurements.PeakPower);
            
    nsam = driver.DeviceSpecific.Measurements.NumberOfSamples;

    [complexdata, numCopied] = driver.DeviceSpecific.Measurements.GetData();
    
    fprintf(1,'Got %d of %d acquired complex data samples at %f per second.\n', numCopied/2, nsam, driver.DeviceSpecific.Measurements.SampleRate);
    nsam = numberPrompt('Number of samples to display\n', 0, numCopied/2, 1, numCopied/2);
    if(nsam>0)
        disp('   #       I         Q')
        for n=1:nsam
           fprintf(1, '   %d	%f    %f\n', n, complexdata(2*n-1), complexdata(2*n));
        end
    end
end