/******************************************************************************
*  Copyright (c) 2010 Agilent Technologies Inc.  All rights reserved.
*
* You have a royalty-free right to use, modify, reproduce and distribute
* the sample application files (and/or any modified version) in any way
* you find useful, provided that you agree that Agilent Technologies has no
* warranty, obligations or liability for any sample application files.
*
* Agilent Technologies provides programming examples for illustration only,
* This sample program assumes that you are familiar with the programming
* language being demonstrated and the tools used to create and debug
* procedures. Agilent Technologies support engineers can help explain the
* functionality of Agilent Technologies software components and associated
* commands, but they will not modify these samples to provide added
* functionality or construct procedures to meet your specific needs.
*****************************************************************************/
///
/// Agilent IVI-C Driver Example Program
/// 
/// Initializes the driver, reads a few Identity interface
/// properties, and initiates instrument specific functionality.
/// 
/// Runs in simulation mode without an instrument.
/// 

#include <stdio.h>
#include <malloc.h>
#include "AgM9392.h"


int binaryPrompt(char* label, int def);
double numberPrompt(char* label, double min, double max, double multiplier, double defval);
ViStatus setupAcquisition(ViSession session);
ViStatus acquireMeasurement(ViSession session, int timeout);
void reportError(char *routine, ViSession session, ViStatus status);

void main(int argc, char **argv)
{
	ViStatus status;
	ViSession session;
	ViChar str[128];
	
	ViBoolean simulate;

	ViBoolean idQuery = VI_TRUE;
	ViBoolean reset   = VI_TRUE;

	/*
	* You can change the code below to hard-code your resource descriptor and simulation mode, or you can pass the resource
    * descriptor in the command line when you run the example program to use that hardware in non-simulation mode.
	*
    * The resource descriptor is the same value as the Selected Instrument field of the SFPs (Soft Front Panel)
    * connection dialog.   This can be a comma or semicolon separated list of module addresses such as
    *     PXI10::15::0::INSTR;PXI10::16::0::INSTR;PXI10::17::0::INSTR;PXI10::18::0::INSTR
    * or the name of a saved Instrument Connection created by the SFP such as
    *     PreselectedVSA
    * Refer to the SFP Help topic Connect to Instruments and Modules for more details.
	*/

	// Pass in a command line argument as the resource descriptor, if none, will default
	ViPChar resource = "PXI20::13::INSTR";
	ViPChar options = "QueryInstrStatus=true, Simulate=true, DriverSetup= Model=, Trace=false";
	if (argc > 1)
	{
		resource = argv[1];
    	options  = "QueryInstrStatus=true, Simulate=false, DriverSetup= Model=, Trace=false";
	}

	// Initialize the driver.  See driver help topic "Initializing the IVI-C Driver" for additional information
	status = AgM9392_InitWithOptions(resource, idQuery, reset, options, &session);
	if(status)
	{
		// Initialization failed
		reportError("AgM9392_InitWithOptions",session,status);
		return;
	}
	printf("Driver Initialized \n");

	// Read and output a few attributes
	status = AgM9392_GetAttributeViString(session, "", AGM9392_ATTR_SPECIFIC_DRIVER_PREFIX, 127, str);
	if(status){
		reportError("AgM9392_GetAttributeViString",session,status);
		return;
	}
	printf("DRIVER_PREFIX:      %s\n", str);
	status = AgM9392_GetAttributeViString(session, "", AGM9392_ATTR_SPECIFIC_DRIVER_REVISION, 127, str);
	if(status){
		reportError("AgM9392_GetAttributeViString",session,status);
		return;
	}
	printf("DRIVER_REVISION:    %s\n", str);
	status = AgM9392_GetAttributeViString(session, "", AGM9392_ATTR_SPECIFIC_DRIVER_VENDOR, 127, str);
	if(status){
		reportError("AgM9392_GetAttributeViString",session,status);
		return;
	}
	printf("DRIVER_VENDOR:      %s\n", str);
	status = AgM9392_GetAttributeViString(session, "", AGM9392_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 127, str);
	if(status){
		reportError("AgM9392_GetAttributeViString",session,status);
		return;
	}
	printf("DRIVER_DESCRIPTION: %s\n", str);
	status = AgM9392_GetAttributeViString(session, "", AGM9392_ATTR_INSTRUMENT_MODEL, 127, str);
	if(status){
		reportError("AgM9392_GetAttributeViString",session,status);
		return;
	}
	printf("INSTRUMENT_MODEL:   %s\n", str);
	status = AgM9392_GetAttributeViString(session, "", AGM9392_ATTR_INSTRUMENT_FIRMWARE_REVISION, 127, str);
	if(status){
		reportError("AgM9392_GetAttributeViString",session,status);
		return;
	}
	printf("FIRMWARE_REVISION:  %s\n", str);
	status = AgM9392_GetAttributeViBoolean(session, "", AGM9392_ATTR_SIMULATE, &simulate);
	if(status){
		reportError("AgM9392_GetAttributeViBoolean",session,status);
		return;
	}
	printf("SIMULATE:	    %s\n", (simulate == VI_TRUE)?"True":"False");
	printf("\n");
    

	status = setupAcquisition(session);
	if(status)
	{
		reportError("setupAcquisition",session,status);
		return;
	}
	if (binaryPrompt("Initiate", 1))
	{
		status = acquireMeasurement(session, 1000);
		if(status)
		{
			reportError("acquireMeasurement",session,status);
			return;
		}
	}
	
    /*
     * Polling the instrument for queued errors is not necessary.  No errors are queued.
     * rather, each of the above driver calls would have thrown an exception if an error occurred.
     * Any thrown exceptions would be handled in the catch clause below:
	 */

	// Close the driver
	printf("\n");
	status = AgM9392_close(session);
	if(status)
	{
		reportError("AgM9392_close",session,status);
		return;
	}
	else
		printf("Driver Closed \n");

	printf("\nDone - Press Enter to Exit");
	getchar();  
}

int binaryPrompt(char* label, int def)
{
	char answer[1000];
	while(1)
	{
		printf("%s? (enter 'yes' or 'no'. default=%s): ", label,(def!=0)?"yes":"no");
		gets_s(answer,1000);
		switch(answer[0])
		{
		case 0:
			return def;
		case 'y':
		case 'Y':
			return 1;
		case 'n':
		case 'N':
			return 0;
		default:
			printf("Please respond 'yes' or 'no':\n");
		}
	}
}

double numberPrompt(char* label, double min, double max, double multiplier, double defval)
{
	char answer[1000];
	double value;
	while (1)
	{
		printf("%s? (enter a value between %lf and %lf. default=%lf): ", 
			label, min/multiplier, max/multiplier,defval);
		gets_s(answer,1000);
		if(answer[0] > 0)
		{
			sscanf_s(answer,"%lf",&value);
		}
		else
		{
			value = defval;
		}
		value *= multiplier;
		if ((value >= min) && (value <= max)) return value;
		printf("     Response must be between %lf and %lf!\n",min/multiplier,max/multiplier);
	}            
}

ViStatus setupAcquisition(ViSession session)
{
	ViStatus status;
	double PRESELECTOR_MIN_FREQ = 2.9e9;
	char prompt[1024];

	ViReal64 maxFrequency, minFrequency, frequency;
	ViBoolean preselectorEnabled;
	ViReal64 minPower, maxPower, power;
	ViReal64 maxBandwidth, bandwidth;
	ViReal64 maxTime, acquisitionTime;
	ViBoolean fieldCalibrationEnabled;
	ViInt32 triggerSource = AGM9392_VAL_TRIGGER_SOURCE_IMMEDIATE;

	status = AgM9392_GetAttributeViReal64 ( session, "", AGM9392_ATTR_MAX_FREQUENCY, &maxFrequency );
	if (status) return status;
	status = AgM9392_GetAttributeViReal64 ( session, "", AGM9392_ATTR_MIN_FREQUENCY, &minFrequency );
	if (status) return status;

	sprintf_s(prompt,1024,"Use Preselector(%lf-%lfGHz)", PRESELECTOR_MIN_FREQ/1e9, (double)maxFrequency/1e9);
	preselectorEnabled = (binaryPrompt(prompt,0)!=0)?VI_TRUE:VI_FALSE;
	frequency = (ViReal64)numberPrompt("Frequency (GHz)",minFrequency,maxFrequency,1e9,8.0);
	if((frequency < PRESELECTOR_MIN_FREQ) && (preselectorEnabled==VI_TRUE))
	{
		printf("     DISABLING preselector (frequency too low)\n");
		preselectorEnabled = 0;
	}
	status = AgM9392_SetAttributeViBoolean (session,"", AGM9392_ATTR_PRESELECTOR_ENABLED,  preselectorEnabled );
	if (status) return status;
	status = AgM9392_SetAttributeViReal64 ( session,"", AGM9392_ATTR_FREQUENCY,  frequency );
	if (status) return status;


	status = AgM9392_GetAttributeViReal64 ( session, "", AGM9392_ATTR_MIN_POWER, &minPower );
	if (status) return status;
	status = AgM9392_GetAttributeViReal64 ( session, "", AGM9392_ATTR_MAX_POWER, &maxPower );
	if (status) return status;
	power = numberPrompt("Power (dBm)", minPower,maxPower,1,-10);

	status = AgM9392_GetMaxBandwidth ( session, frequency, preselectorEnabled, &maxBandwidth );
	if (status) return status;
	bandwidth = numberPrompt("bandwidth (kHz)", 0, maxBandwidth, 1000, 250);

	maxTime = 10; // the maximum actually depends on other setup conditions, please refer to the documentation
	acquisitionTime = numberPrompt("Acquisition Time(ms)", 0, maxTime, 0.001, 10);

	fieldCalibrationEnabled = (binaryPrompt("Calibration", 1)!=0) ? VI_TRUE : VI_FALSE;

	status = AgM9392_SetAttributeViBoolean (session, "", AGM9392_ATTR_FIELD_CALIBRATION_ENABLED, fieldCalibrationEnabled );
	if (status) return status;

	status = AgM9392_ConfigureAcquisition (session, preselectorEnabled, frequency, power, bandwidth, acquisitionTime, triggerSource );
	return status;
}


ViStatus acquireMeasurement(ViSession session, int timeout)
{
	ViStatus status;
	ViInt32 timeItTook;
	ViReal64 rmsPower, peakPower;
	ViReal64 sampleRate;
	ViInt32 nsam;

	ViInt32 numCopied = 0;
	ViInt32 actualSize = 0;
	ViReal32 *complexData = 0;
	int t;
	
	printf("\n");
	printf("Initiate measurement...\n");

	status = AgM9392_Initiate ( session );
	if (status) return status;
	status = AgM9392_WaitForInitiateComplete ( session, (ViInt32)timeout, &timeItTook );
	if (status) return status;

	printf("Arming...\n");
	status = AgM9392_Arm ( session );
	if (status) return status;
	status = AgM9392_WaitForData ( session, (ViInt32)timeout, &timeItTook );
	if (status) return status;

	status = AgM9392_GetRmsPower ( session, &rmsPower );
	if (status) return status;
	printf("RMS Power = %lf dBm.\n", rmsPower);
	status = AgM9392_GetPeakPower ( session, &peakPower );
	if (status) return status;
	printf("Peak Power = %lf dBm.\n", peakPower);
	status = AgM9392_GetSampleRate ( session, &sampleRate );
	if (status) return status;
	status = AgM9392_GetNumberOfSamples ( session, &nsam );
	if (status) return status;
	
	complexData = malloc(2*sizeof(ViReal32)*nsam);//new ViReal32[2*nsam];
	status = AgM9392_GetData (session, nsam*2, complexData, &actualSize, &numCopied );
	if(status)
	{
		free(complexData);
		return status;
	}
	
	printf("Acquired %d complex data samples at %lf samples per second.", nsam, sampleRate);	
	nsam = (long)numberPrompt("Number of samples to display", 0, (double)numCopied/2, 1, (double)numCopied/2);
	if(nsam>0)
	{
		printf("	#		I		Q\n");
		for (t = 0; t < nsam; t ++)
		{
			printf("   %d      %f      %f\n",t,complexData[2*t],complexData[2*t+1]);   
		}
	}
	free(complexData);
	return status;
}

void reportError(char *routine, ViSession session, ViStatus status)
{
	ViInt32 ErrorCode;
	ViChar ErrorMessage[256];
	AgM9392_GetError(session, &ErrorCode, 255, ErrorMessage);
	printf("** %s() Error: %d, %s\n",routine, ErrorCode, ErrorMessage);
	printf("\nAbort - Press Enter to Exit");
	getchar();  
}

