/******************************************************************************
*  Copyright (c) 2010 Agilent Technologies Inc.  All rights reserved.
*
* You have a royalty-free right to use, modify, reproduce and distribute
* the sample application files (and/or any modified version) in any way
* you find useful, provided that you agree that Agilent Technologies has no
* warranty, obligations or liability for any sample application files.
*
* Agilent Technologies provides programming examples for illustration only,
* This sample program assumes that you are familiar with the programming
* language being demonstrated and the tools used to create and debug
* procedures. Agilent Technologies support engineers can help explain the
* functionality of Agilent Technologies software components and associated
* commands, but they will not modify these samples to provide added
* functionality or construct procedures to meet your specific needs.
*****************************************************************************/

using System;
using AgM9392Lib;

namespace CS_BasicMeasurement
{
	/// <summary>
    ///
    /// Agilent IVI-C Driver Example Program
    /// 
    /// Initializes the driver, reads a few Identity interface
    /// properties, and initiates instrument specific functionality.
    /// Runs in simulation mode without an instrument.
    /// 
	/// Requires a COM reference to the driver's type library.
	/// 
	/// </summary>

    public class App
	{

        private static bool binaryPrompt(string label, bool def)
        {
            while(true)
            {
                Console.Write("{0}? (enter 'yes' or 'no'. default={1}): ", label,(def)?"yes":"no");
                string answer = Console.ReadLine().ToLower();
                if(answer.Length==0) return def;
                if(answer.StartsWith("n")) return false;
                if(answer.StartsWith("y"))return true;
                Console.WriteLine("Please respond 'yes' or 'no':");
            }
        }

        private static double numberPrompt(string label, double min, double max, double multiplier, double defval)
        {
            while (true)
            {
                Console.Write("{0}? (enter a value between {1} and {2}. default={3}): ", 
                    label, min/multiplier, max/multiplier,defval);
                string answer = Console.ReadLine();
                double value = (answer.Length > 0) ? double.Parse(answer) : defval;
                value *= multiplier;
                if ((value >= min) && (value <= max)) return value;
                Console.WriteLine("     Response must be between {0} and {1}!",min/multiplier,max/multiplier);
            }            
        }

        private static void setupAcquisition(AgM9392 driver)
        {
            double PRESELECTOR_MIN_FREQ = 2.9e9;
            string prompt = String.Format("Use Preselector({0}-{1}GHz)", PRESELECTOR_MIN_FREQ/1e9,
                                          driver.Acquisition.MaxFrequency/1e9);
            driver.Acquisition.PreselectorEnabled = binaryPrompt(prompt,false);
            driver.Acquisition.Frequency = numberPrompt("Frequency (GHz)", driver.Acquisition.MinFrequency,
                                                        driver.Acquisition.MaxFrequency,1e9,8.0);
            if((driver.Acquisition.Frequency < PRESELECTOR_MIN_FREQ)
                && driver.Acquisition.PreselectorEnabled)
            {
                Console.WriteLine("     DISABLING preselector (frequency too low)");
                driver.Acquisition.PreselectorEnabled = false;
            }
            driver.Acquisition.Power = numberPrompt("Power (dBm)", driver.Acquisition.MinPower,
                                                    driver.Acquisition.MaxPower, 1, -10);
            driver.Acquisition.Bandwidth = numberPrompt("bandwidth (kHz)", 0, driver.Acquisition.MaxBandwidth, 1000, 250);
            double maxTime = 10; // the maximum actually depends on other setup conditions, please refer to the documentation
            driver.Acquisition.Time = numberPrompt("Acquisition Time(ms)", 0, maxTime, 0.001, 10);
            driver.Calibration.FieldCalibration.Enabled = binaryPrompt("Calibration",true);
        }

        private static void acquireMeasurement(AgM9392 driver, int timeout)
        {
            Console.WriteLine("");
            Console.WriteLine("Initiate measurement...");
            driver.Measurements.Initiate();
            driver.Measurements.WaitForInitiateComplete(timeout);
            Console.WriteLine("Arming...");
            driver.Measurements.Arm();



            driver.Measurements.WaitForData(timeout);
 
            Console.WriteLine("RMS Power = {0} dBm.", driver.Measurements.RmsPower);
            Console.WriteLine("Peak Power = {0} dBm.", driver.Measurements.PeakPower);
            
            int nsam = driver.Measurements.NumberOfSamples;
            int numCopied = 0;
            Array array = Array.CreateInstance(typeof(float), 2 * nsam);
            driver.Measurements.GetData(ref array, ref numCopied);
            Console.WriteLine("Acquired {0} complex data samples at {1} per second.", nsam, driver.Measurements.SampleRate);
            nsam = (int)numberPrompt("Number of samples to display", 0, (double)numCopied/2, 1, (double)numCopied/2);
            Console.WriteLine("   #       I         Q");
            for (int n = 0; n < nsam; n ++)
            {
                Console.WriteLine("   {0}     {1}     {2}", n, array.GetValue(2*n), array.GetValue(2*n+1));   
            }
        }

	    [STAThread]
		public static void Main(string[] args) 
		{
            Console.WriteLine("CS_BasicMeasurement");
            Console.WriteLine();

            AgM9392 driver = null;

            /*
            * You can change the code below to hard-code your resource descriptor and simulation mode, or you can pass the resource
            * descriptor in the command line when you run the example program to use that hardware in non-simulation mode.
            *
            * The resource descriptor is the same value as the Selected Instrument field of the SFPs (Soft Front Panel)
            * connection dialog.   This can be a comma or semicolon separated list of module addresses such as
            *     PXI10::15::0::INSTR;PXI10::16::0::INSTR;PXI10::17::0::INSTR;PXI10::18::0::INSTR
            * or the name of a saved Instrument Connection created by the SFP such as
            *     PreselectedVSA
            * Refer to the SFP Help topic Connect to Instruments and Modules for more details.
            */

            // Pass in a command line argument as the resource descriptor, if none, will default
            string resource = "PXI20::13::INSTR";
            string options = "QueryInstrStatus=true, Simulate=true, DriverSetup= Model=, Trace=false";
            if (args.Length > 0)
            {
                resource = args[0];
                options = "QueryInstrStatus=true, Simulate=false, DriverSetup= Model=, Trace=false";
            }

			try
			{
				// Create driver instance
				driver = new AgM9392(); //Agilent.AgilentDriverShell.Interop.AgilentDriverShell();

				bool idquery = true;
				bool reset   = true;

                // Initialize the driver.  See driver help topic "Initializing the IVI-COM Driver" for additional information
                driver.Initialize(resource, idquery, reset, options);
				Console.WriteLine("Driver Initialized");

				// Print a few IIviDriverIdentity properties
                Console.WriteLine("Identifier:  {0}", driver.Identity.Identifier);
                Console.WriteLine("Revision:    {0}", driver.Identity.Revision);
                Console.WriteLine("Vendor:      {0}", driver.Identity.Vendor);
                Console.WriteLine("Description: {0}", driver.Identity.Description);
                Console.WriteLine("Model:       {0}", driver.Identity.InstrumentModel);
                Console.WriteLine("FirmwareRev: {0}", driver.Identity.InstrumentFirmwareRevision);
                Console.WriteLine("Serial #:    {0}", driver.System.SerialNumber);
                Console.WriteLine("Simulate:    {0}", driver.DriverOperation.Simulate);
                Console.WriteLine("");

                setupAcquisition(driver);
                if (binaryPrompt("Initiate", true))
                {
                    acquireMeasurement(driver, 1000);
                }
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
			}
			finally
			{
				if (driver != null && driver.Initialized)
				{
					// Close the driver
                    driver.Close();
                    Console.WriteLine("");
                    Console.WriteLine("Driver Closed");
				}
			}
            Console.WriteLine("");
            Console.WriteLine("\nDone - Press Enter to Exit");
			Console.ReadLine();
		}
	}
}
