using System;
using System.Windows.Forms;

using AgM9392Lib;

namespace CS_BasicMeasurementGUI
{
    public partial class BasicMeasurement : Form
    {
        private AgM9392 mVsa;
        private readonly double PRESELECTOR_MIN_FREQ = 2.9e9;

        public BasicMeasurement()
        {
            InitializeComponent();

            toolStripStatusLabel1.Text = "Ready (Open the VSA next)";
        }

        private void UpdateLimits()
        {
            if (mVsa != null)
            {
                // Populate the limit controls with the values from the VSA
                minFrequency_MHz.Text = String.Format("{0} MHz", 1.0e-6*mVsa.Acquisition.MinFrequency);
                maxFrequency_GHz.Text = String.Format("{0} GHz", 1.0e-9*mVsa.Acquisition.MaxFrequency);
                minBandwidth_kHz.Text = "---";
                double value = 0;
                mVsa.Acquisition.GetMaxBandwidth(mVsa.Acquisition.Frequency, mVsa.Acquisition.PreselectorEnabled, ref value);
                maxBandwidth_MHz.Text = String.Format("{0} MHz", 1.0e-6*value);
                minPower_dBm.Text = String.Format("{0} dBm", mVsa.Acquisition.MinPower);
                maxPower_dBm.Text = String.Format("{0} dBm", mVsa.Acquisition.MaxPower);
                minAcquisitionTime_ms.Text = "---";
                double maxTime = 10; // the maximum actually depends on other setup conditions, please refer to the documentation
                maxAcquisitionTime_s.Text = String.Format("{0} s", maxTime);

                if (mVsa.Acquisition.Frequency < PRESELECTOR_MIN_FREQ)
                {
                    PreselectorBox.Checked = false;
                    PreselectorBox.Enabled = false;
                    PreselectorLabel.Enabled = false;
                }
                else
                {
                    PreselectorBox.Enabled = true;
                    PreselectorLabel.Enabled = true;
                }
            }
        }

        private void openButton_Click(object sender, EventArgs e)
        {
            toolStripStatusLabel1.Text = "Opening VSA...";

            try
            {
                string options = Options.Text;
                if (simulatedCheck.Checked)
                {
                    options = "Simulate=TRUE," + options;
                }

                // Instantiate and Initialize the M9392A VSA driver 
                mVsa = new AgM9392Class();
                mVsa.Initialize(IOResourceDescriptor.Text, false, false, options);

                // adjust GUI enables...
                simulatedCheck.Enabled = false;
                openButton.Enabled = false;
                closeButton.Enabled = true;
                initiateButton.Enabled = true;

                // The instrument will perform power on self test as it
                // connects to the modules and verifies the configuration.
                // Technically, it will already be in the preset state when
                // the constructor call returns.  For good form, we
                // force the hardware to a known state again.
                mVsa.Utility.Reset();

                // Update limits...
                UpdateLimits();

                toolStripStatusLabel1.Text = "Ready";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error opening VSA", MessageBoxButtons.OK, MessageBoxIcon.Error );

                // Get rid of the M9392A VSA driver and adjust GUI enables...
                mVsa = null;

                simulatedCheck.Enabled = true;
                openButton.Enabled = true;
                closeButton.Enabled = false;
                initiateButton.Enabled = false;

                toolStripStatusLabel1.Text = "Error opening VSA";
            }
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            // Get rid of the M9392A VSA driver and adjust GUI enables...
            mVsa = null;

            simulatedCheck.Enabled = true;
            openButton.Enabled = true;
            closeButton.Enabled = false;
            initiateButton.Enabled = false;

            // Clear limits
            minFrequency_MHz.Text = "";
            maxFrequency_GHz.Text = "";
            minBandwidth_kHz.Text = "";
            maxBandwidth_MHz.Text = "";
            minPower_dBm.Text = "";
            maxPower_dBm.Text = "";
            minAcquisitionTime_ms.Text = "";
            maxAcquisitionTime_s.Text = "";

            toolStripStatusLabel1.Text = "Ready (Open the VSA next)";
            toolStripTriggerTimeout.Visible = false;
            toolStripOverload.Visible = false;
        }

        private void initiateButton_Click(object sender, EventArgs e)
        {
            try
            {
                toolStripStatusLabel1.Text = "Setting VSA parameters...";

                // Clear any existing errors
                mVsa.Status.Clear();

                // Convert settings to base units...
                double frequency = 1.0e9*double.Parse(frequency_GHz.Text);
                double power = double.Parse(power_dBm.Text);
                double bandwidth = 1.0e3*double.Parse(bandwidth_kHz.Text);
                double acquisitionTime = 1.0e-3*double.Parse(acquisitionTime_ms.Text);
                bool usePreselector = PreselectorBox.Checked && frequency >= PRESELECTOR_MIN_FREQ;

                // We can use a single call to set up the receiver with the
                // most common parameters
                mVsa.Acquisition.Configure(
                    usePreselector,
                    frequency, // in Hz
                    power, // in dBm
                    bandwidth, // in Hz
                    acquisitionTime, // in s
                    AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate);

                // The SetupMeasurement call is equivalent to:
                // (only need to do one or the other)
                mVsa.Acquisition.Frequency = frequency; // in Hz
                mVsa.Acquisition.Power = power; // in dBm
                mVsa.Acquisition.Bandwidth = bandwidth; // in Hz
                mVsa.Acquisition.Time = acquisitionTime; // in s
                mVsa.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate;
                mVsa.Calibration.FieldCalibration.Enabled = calEnabled.Checked;

                // Additional parameters if triggering...
                // NOTICE: video and external trigger levels have different units (dBm vs. volts)
                mVsa.Trigger.Timeout = 1000; // in ms
                mVsa.Trigger.Delay = 0; // in s
                mVsa.Trigger.Video.Level = -30; // in dBm
                mVsa.Trigger.Video.Slope = AgM9392VideoTriggerSlopeEnum.AgM9392VideoTriggerSlopePositive;
                mVsa.Trigger.External.Level = 0; // in volts
                mVsa.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopeNegative;

                // There are convenience methods, Configure, that are the same as setting
                // the trigger attributes individually (only need to do one or the other)
                mVsa.Trigger.Video.Configure(-30, AgM9392VideoTriggerSlopeEnum.AgM9392VideoTriggerSlopePositive);
                mVsa.Trigger.External.Configure(0, AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopeNegative);

                // The driver now has all the information it needs to make a
                // measurement.  Nothing has been written to the hardware, and
                // the requested settings have not been checked against the
                // available hardware.  This all happens when we call the
                // Initiate routine, which selects the specific hardware path
                // to be used in the measurement, and starts the hardware
                // slewing to the new state.
                try
                {
                    toolStripStatusLabel1.Text = "Initiate measurement...";
                    mVsa.Measurements.Initiate();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                    // If an error occurred, there was a problem meeting the
                    // requested settings, and we need to read the messages in
                    // the message queue to find out what is wrong.
                    while (mVsa.Status.NumberOfErrors > 0)
                    {
                        int errorCode = 0;
                        string errorMessage = "No Error";
                        mVsa.Utility.ErrorQuery(ref errorCode, ref errorMessage);
                        MessageBox.Show(String.Format("{0} : {1}", errorCode, errorMessage),
                                        "Reported Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }

                // Since we dont have to synchronize to any other devices or
                // instruments, we go ahead and call Arm().  In Free Run mode,
                // the Arm() method waits until the hardware is ready, then
                // begins the acquisition as soon as possible.
                toolStripStatusLabel1.Text = "Arming...";
                mVsa.Measurements.Arm();

                // Now that acquisition has begun, we take a moment to allocate
                // the buffer to hold the complex measured data, which is 32 bit
                // floats, interleaved real and imag. The user will probably
                // also want to know what sample rate the data was taken at  the
                // sample rate varies with the requested bandwidth.  Use
                //  GetCurrentSampleRate() to query the sample rate.
                int numSamples = mVsa.Measurements.NumberOfSamples;
                Array myData = new float[2 * numSamples];

                // Now we block until data is available and has been copied into
                // our buffer
                Int32 numSamplesReturned = 0;
                toolStripStatusLabel1.Text = "Waiting for data...";
                mVsa.Measurements.GetData(
                    ref myData, // the data buffer
                    ref numSamplesReturned); // # of samples actually copied to myData

                // Query the measured power of the signal
                double rmsPower = mVsa.Measurements.RmsPower;
                double peakPower = mVsa.Measurements.PeakPower;

                // Lastly, we check to see that all went well with the
                // acquisition.  Unlike a SCPI instrument, checking status is
                // very low overhead and should be done frequently.  We want
                // to check for messages in the message queue, as well as
                // specifically check for overload
                while (mVsa.Status.NumberOfErrors > 0 )
                {
                    int errorCode = 0;
                    string errorMessage = "No Error";
                    mVsa.Utility.ErrorQuery(ref errorCode, ref errorMessage);
                    MessageBox.Show(String.Format("{0} : {1}", errorCode, errorMessage),
                                    "Reported Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                // Now check for overload
                toolStripOverload.Visible = mVsa.Measurements.DidAcquisitionOverload;

                // Now check if the trigger timed out
//                 toolStripTriggerTimeout.Visible = mVsa.Measurements.DidTriggerTimeout;

                // Display the data
                rmsPower_dBm.Text = rmsPower.ToString();
                peakPower_dBm.Text = peakPower.ToString();

                // Limit the displayed points (the list control does not effectively handle the maximum)
                int displayedPoints = Math.Min(numSamples, 10000);
                dataList.Items.Clear();
                for (int j = 0; j < displayedPoints; j++)
                {
                    string iq = string.Format("{0} : {1}", myData.GetValue(2*j), myData.GetValue(2*j+1));
                    dataList.Items.Add(iq);
                }
                if (displayedPoints < numSamples)
                {
                    dataList.Items.Add(String.Format("...{0} not displayed...", numSamples - displayedPoints));
                }

                toolStripStatusLabel1.Text = "Ready";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error invoking measurement", MessageBoxButtons.OK, MessageBoxIcon.Error);

                toolStripStatusLabel1.Text = "Error invoking measurement";
            }
        }

        private void frequency_GHz_Validated(object sender, EventArgs e)
        {
            // Some limits depend on frequency -- so update the limits when that changes
            if (mVsa != null)
            {
                try
                {
                    double frequency = 1.0e9 * double.Parse(frequency_GHz.Text);
                    mVsa.Acquisition.Frequency = frequency;
                }
                catch (Exception)
                {
                    MessageBox.Show("Error parsing Frequency", "Parse Error", MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);

                    frequency_GHz.Text = String.Format("{0}", 1.0e-9*mVsa.Acquisition.Frequency);
                }
            }
            UpdateLimits();
        }

        private void bandwidth_kHz_Validated(object sender, EventArgs e)
        {
            // Some limits depend on bandwidth -- so update the limits when that changes
            if (mVsa != null)
            {
                try
                {
                    double bandwidth = 1.0e3 * double.Parse(bandwidth_kHz.Text);
                    mVsa.Acquisition.Bandwidth = bandwidth;
                }
                catch (Exception)
                {
                    MessageBox.Show("Error parsing Bandwidth", "Parse Error", MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);

                    bandwidth_kHz.Text = String.Format("{0}", 1.0e-3 * mVsa.Acquisition.Bandwidth);
                }
            }
            UpdateLimits();
        }

        private void power_dBm_Validated(object sender, EventArgs e)
        {
            // No limits depend on power -- so wait until Initiate() to parse
        }

        private void acquisitionTime_ms_Validated(object sender, EventArgs e)
        {
            // No limits depend on acquisition time -- so wait until Initiate() to parse
        }

        private void PreselectorBox_CheckedChanged(object sender, EventArgs e)
        {
            // Some limits depend on preselector -- so update the limits when that changes
            if (mVsa != null)
            {
                try
                {
                    bool usePreselector = PreselectorBox.Checked && mVsa.Acquisition.Frequency >= PRESELECTOR_MIN_FREQ;
                    mVsa.Acquisition.PreselectorEnabled = usePreselector;
                }
                catch (Exception)
                {
                    MessageBox.Show("Error processing preselector", "Parse Error", MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);
                }
            }
            UpdateLimits();
        }
    }
}
