// Field Calibration

// The M9392 Microwave VSA is constructed out of PXI modules.  Any time the front panel cables 
// are disconnected and reconnected, the power accuracy will suffer.  Similarly, if one of the
// modules is swapped out, there is going to be a small perturbation to the path loss, and
// hence the power accuracy.
//
// To provide best power accuracy, the M9392 should be calibrated in the field.  To perform 
// this calibration, you need a microwave source that can cover the entire 40 MHz to 26.5 GHz,
// and a NIST traceable power meter and sensor.
//
// The cal procedure is to first use the GetCalFreqList() query to find out the complete 
// frequency list the M9392 needs to be calibrated over.  The actual list generated will 
// depend on which modules are present in the instrument.
//
// The user (or the automating program) then needs to loop over that frequency list, setting 
// the source to each frequency at a nominal -10 dBm, measuring the actual power delivered 
// to the M9392, and calling back to the M9392 using the CalThisFreq() method, which causes 
// the M9392 to calibrate all paths that use that input frequency.
//
// There are two ways to provide the calibrated input power to the M9392.  The easiest way is 
// to connect a power meter directly to the microwave source output, and step through the 
// frequency list, measuring the source output power at each frequency.  Then the user connects 
// the microwave source to the M9392 input, and we sequence the source through the exact same 
// list (same frequencies, same powers), relying on the repeatability and leveling of the 
// source to provide the same power to the M9392 as to the power meter.
//
// The more accurate approach is to use a two resister power splitter and and a dual power 
// meter.  The common terminal of the splitter is connected to the source output.  One power 
// sensor is permanently connected to one leg of the splitter.  To calibrate the splitter, 
// we step through the frequency list with a second power sensor on the other leg of the
// splitter, recording the difference between the power meter readings as the splitter 
// calibration.  Then we disconnect the second power sensor and connect the M9392 receiver. 
// At each frequency in the list, cache the input power recorded by the permanent sensor, 
// apply the splitter correction, and pass the resulting power into CalThisFreq().
//
// It is recommended that the power level at the M9392 input is between -10 and -15 dBm.   
// If a power splitter is used, or if some lossy fixturing is being included in the 
// calibration, it is recommended that a first order correction of the source power be done
// to keep the power in the -10 to -15 dBm range.
//
// The customer can include parts of the test fixturing, cabling and switching in the 
// calibration.  The power at the M9392 input should be maintained between -10 and -15 dBm. 
// Notice that the frequency spacing of the calibration is relatively coarse  the calibration
// will not work well for frequency selective devices that have significant change in insertion
// loss across a 25 MHz step.
//
// For manual operation, the M9392 Soft Front Panel will guide the customer through the
// calibration.  The customer must have one of the following microwave sources:
// 	E8257D PSG covering 26.5 GHz (Analog/CW)
// 	E8267D PSG covering 26.5 GHz (Vector Signal Generator)
// And one of the following Power Meters
// 	N1913A EPM Single Channel Power Meter 
// 	N1914A EPM Dual Channel Power Meter
// And the following power sensor:
// 	N8485A 10 MHz  26.5 GHz Thermocouple -35 to +20 dBm  ** recommended **

// This example below demonstrates how to do the simpler calibration, with no power splitter.

using System;
using System.Threading;
using System.Windows.Forms;
using AgM9392Lib;

namespace CS_FieldCalibration
{
    public partial class FieldCalibration : Form
    {
        private const double DESIRED_POWER = -10;  // desired power at VSA for calibration
        private const double MINIMUM_POWER = -12.5; // minimum power at VSA for calibration
        private const double MAXIMUM_POWER = 10; // maximum power to be requested from source

        private AgM9392 mVsa;
        private bool mAbort;
        private readonly Source mSource = new Source();
        private readonly PowerMeter mPowerMeter = new PowerMeter();

        public FieldCalibration()
        {
            InitializeComponent();
        }

        private void simulatedSourceCheck_CheckedChanged(object sender, EventArgs e)
        {
            sourceResourceDescriptor.Enabled = !simulatedSourceCheck.Checked;
        }

        private void simulatedMeterCheck_CheckedChanged(object sender, EventArgs e)
        {
            meterResourceDescriptor.Enabled = !simulatedMeterCheck.Checked;

        }

        private void simulatedVsaCheck_CheckedChanged(object sender, EventArgs e)
        {
            vsaResourceDescriptor.Enabled = !simulatedVsaCheck.Checked;
        }

        private void openButton_Click(object sender, EventArgs e)
        {
            toolStripStatusLabel1.Text = "Opening VSA...";
            Cursor.Current = Cursors.WaitCursor;

            try
            {
                string options = simulatedVsaCheck.Checked ? "Simulate=TRUE" : "";

                // Instantiate and Initialize the M9392A VSA driver 
                mVsa = new AgM9392Class();
                mVsa.Initialize(vsaResourceDescriptor.Text, false, false, options);

                // adjust GUI enables...
                simulatedVsaCheck.Enabled = false;
                simulatedSourceCheck.Enabled = false;
                simulatedMeterCheck.Enabled = false;
                vsaResourceDescriptor.Enabled = false;
                sourceResourceDescriptor.Enabled = false;
                meterResourceDescriptor.Enabled = false;
                openButton.Enabled = false;
                closeButton.Enabled = true;
                previewButton.Enabled = true;
                calibrateButton.Enabled = true;
                abortButton.Enabled = false;

                // The instrument will perform power on self test as it
                // connects to the modules and verifies the configuration.
                // Technically, it will already be in the preset state when
                // the constructor call returns.  For good form, we
                // force the hardware to a known state again.
                mVsa.Utility.Reset();

                // Open the source
                mSource.Open(simulatedSourceCheck.Checked, sourceResourceDescriptor.Text);

                // Open the power meter
                mPowerMeter.Open(simulatedMeterCheck.Checked, meterResourceDescriptor.Text);

                toolStripStatusLabel1.Text = "Ready";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error opening instrument(s)", MessageBoxButtons.OK, MessageBoxIcon.Error);

                // Close I/O
                CloseInstruments();

                // Update GUI
                ShowClosedIO();

                toolStripStatusLabel1.Text = "Error opening instrument(s).";
            }
            Cursor.Current = Cursors.Default;
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            // Close I/O
            CloseInstruments();

            // Update GUI
            ShowClosedIO();

            Cursor.Current = Cursors.Default;
        }

        private void FieldCalibration_FormClosing(object sender, FormClosingEventArgs e)
        {
            // When the form is closing, be sure the devices are closed
            // to release any VISA locks
            CloseInstruments();

            // Don't worry about updating the GUI -- we're exiting!
        }

        /// <summary>
        /// Close all instrument connections
        /// </summary>
        private void CloseInstruments()
        {
            try
            {
                // Get rid of the M9392A VSA driver and adjust GUI enables...
                mVsa = null;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error while closing the VSA instrument connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            try
            {
                // Close the source
                mSource.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error while closing the Source instrument connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            try
            {
                // Close the power meter
                mPowerMeter.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error while closing the Power Meter instrument connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Update the GUI controls to indicate that I/O is closed
        /// </summary>
        private void ShowClosedIO()
        {
            simulatedVsaCheck.Enabled = true;
            simulatedSourceCheck.Enabled = true;
            simulatedMeterCheck.Enabled = true;
            vsaResourceDescriptor.Enabled = !simulatedVsaCheck.Checked;
            sourceResourceDescriptor.Enabled = !simulatedSourceCheck.Checked;
            meterResourceDescriptor.Enabled = !simulatedMeterCheck.Checked;
            openButton.Enabled = true;
            closeButton.Enabled = false;
            previewButton.Enabled = false;
            calibrateButton.Enabled = false;
            abortButton.Enabled = false;

            frequencyList.Items.Clear();
            status.Text = "";
            frequency_MHz.Text = "";
            sourcePower_dBm.Text = "";
            measuredPower_dBm.Text = "";

            toolStripStatusLabel1.Text = "Ready (Open the VSA next)";
            toolStripTriggerTimeout.Visible = false;
            toolStripOverload.Visible = false;            
        }

        private void previewButton_Click(object sender, EventArgs e)
        {
            try
            {
                // This method is not part of the calibration, but simply
                // lets the user peruse the frequencies the instrument
                // wants to be calibrated at
                Array frequencies = new double[0];
                mVsa.Calibration.FieldCalibration.GetFrequencyList(ref frequencies);

                frequencyList.Items.Clear();
                foreach (double frequency in frequencies)
                {
                    frequencyList.Items.Add(String.Format("{0} MHz", 1.0e-6 * frequency));                    
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error retrieving calibration frequencies", MessageBoxButtons.OK, MessageBoxIcon.Error);

                toolStripStatusLabel1.Text = "Error retrieving calibration frequencies";
            }
        }

        private void abortButton_Click(object sender, EventArgs e)
        {
            // Just set a flag that calibrateButton_Click() polls for
            mAbort = true;
        }

        private void calibrateButton_Click(object sender, EventArgs e)
        {
            try
            {
                // Disable GUI controls
                previewButton.Enabled = false;
                calibrateButton.Enabled = false;
                closeButton.Enabled = false;
                abortButton.Enabled = true;

                // Optionally perform a cal/zero of the power meter
                toolStripStatusLabel1.Text = "Calibrate power meter";
                DialogResult result = MessageBox.Show(
                    "Connect the power meter to the power reference and click 'Yes' to calibrate the power meter.\nClick 'No' to skip power meter calibration", "User Action Required", 
                    MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                if (result == DialogResult.Cancel)
                {
                    status.Text = "Calibration aborted";
                    toolStripStatusLabel1.Text = "Calibration aborted";
                    return;
                }
                if (result == DialogResult.Yes)
                {
                    mPowerMeter.Calibrate();
                    MessageBox.Show("Wait until the power meter has finished calibrating then click 'OK'",
                                    "User Action Required", MessageBoxButtons.OK);
                    CheckForErrors();
                }

                // Instruct the user to connect source to power meter
                toolStripStatusLabel1.Text = "Connect power meter";
                if (MessageBox.Show("Connect source to power meter", "User Action Required", MessageBoxButtons.OKCancel,
                                MessageBoxIcon.Exclamation) == DialogResult.Cancel)
                {
                    status.Text = "Calibration aborted";
                    toolStripStatusLabel1.Text = "Calibration aborted";
                    return;
                }

                // Retrieve the list of calibration frequencies 
                Array frequencies = new double[0];
                mVsa.Calibration.FieldCalibration.GetFrequencyList(ref frequencies);

                // Create the source power and measured power arrays
                double[] sourcePower = new double[frequencies.Length];
                double[] measuredPower = new double[frequencies.Length];

                // Initially, assume no loss...
                double power = DESIRED_POWER;

                // Especially with themocouple sensors, the first setup may take
                // quite a while to settle since the source is initially off.  So
                // set up the first frequency/power pair and wait 5 seconds...
                if (frequencies.Length > 0)
                {
                    double frequency = (double) frequencies.GetValue(0);
                    SetSource(frequency, power);
                    Thread.Sleep(5000);

                    // Now measure the initial power...
                    double measured = MeasurePower(frequency, power);

                    // Adjust the next source power to target DESIRED_POWER
                    if (Math.Abs(DESIRED_POWER - measured) > 0.1)
                    {
                        power += DESIRED_POWER - measured;
                        power = Math.Min(power, MAXIMUM_POWER);
                    }

                    // Check for errors...
                    CheckForErrors();
                }

                // Loop over the frequencies and measure the actual
                // power at the end of the cable (where the power meter
                // is connected)
                toolStripStatusLabel1.Text = "Measuring power";
                status.Text = "Measuring power";
                for (int index = 0; index < frequencies.Length; index++)
                {
                    double frequency = (double)frequencies.GetValue(index);

                    // Set the source frequency/power
                    SetSource(frequency, power);

                    // Measure the power ... want between DESIRED_POWER and MINIMUM_POWER
                    double measured = MeasurePower(frequency, power);
                    if (measured < MINIMUM_POWER)
                    {
                        // Too small!  Try one more time with correction at the source
                        power += DESIRED_POWER - measured;
                        power = Math.Min(power, MAXIMUM_POWER);

                        // Set the source frequency/power
                        SetSource(frequency, power);

                        // Measure again
                        measured = MeasurePower(frequency, power);
                    }

                    // Display measured power
                    measuredPower_dBm.Text = string.Format("{0:0.000} dBm", measured);

                    // Cache the source and measured powers
                    sourcePower[index] = power;
                    measuredPower[index] = measured;

                    // Adjust the next source power to target DESIRED_POWER
                    if (Math.Abs(DESIRED_POWER - measured) > 0.1)
                    {
                        power += DESIRED_POWER - measured;
                        power = Math.Min(power, MAXIMUM_POWER);
                    }

                    // Check for errors...
                    CheckForErrors();

                    // Check for abort
                    if (mAbort)
                    {
                        mAbort = false;
                        if (MessageBox.Show("Are you sure you want to abort the calibration?", "Abort Calibration?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                        {
                            status.Text = "Calibration aborted";
                            toolStripStatusLabel1.Text = "Calibration aborted";
                            return;
                        }                        
                    }
                }

                // Instruct the user to connect source to M9392A
                if (MessageBox.Show("Connect source to M9392A", "User Action Required", MessageBoxButtons.OKCancel,
                                MessageBoxIcon.Exclamation) == DialogResult.Cancel)
                {
                    status.Text = "Calibration aborted";
                    toolStripStatusLabel1.Text = "Calibration aborted";
                    return;
                }

                // Loop over the frequencies (again) and tell the M9392A
                // to calibrate at each frequency with the power measured
                // by the power meter in the previous loop
                toolStripStatusLabel1.Text = "Performing calibration";
                status.Text = "Performing calibration";
                for (int index = 0; index < frequencies.Length; index++)
                {
                    double frequency = (double)frequencies.GetValue(index);

                    // Set the source frequency/power
                    SetSource(frequency, sourcePower[index]);

                    // Display the previously measured power
                    measuredPower_dBm.Text = string.Format("{0:0.000} dBm", measuredPower[index]);

                    // Calibrate... the first time this is executed the VSA enters
                    // "calibration mode" and will remain in this mode until Cancel or Save.
                    // Calls to Measurements.Initiate() will generate errors while in calibration mode.
                    mVsa.Calibration.FieldCalibration.CalThisFrequency(frequency, measuredPower[index]);

                    // Check for errors...
                    try
                    {
                        CheckForErrors();
                    }
                    catch (Exception)
                    {
                        // Need to tell VSA to exit calibration mode
                        mVsa.Calibration.FieldCalibration.Cancel();
                        throw;
                    }

                    // Check for abort
                    if (mAbort)
                    {
                        mAbort = false;
                        if (MessageBox.Show("Are you sure you want to abort the calibration?", "Abort Calibration?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                        {
                            status.Text = "Calibration aborted";
                            toolStripStatusLabel1.Text = "Calibration aborted";
                            // Need to tell VSA to exit calibration mode
                            mVsa.Calibration.FieldCalibration.Cancel();
                            return;
                        }
                    }
                }

                // Done!  Save the results
                toolStripStatusLabel1.Text = "Saving calibration";
                mVsa.Calibration.FieldCalibration.Save();
                toolStripStatusLabel1.Text = "Ready";
                status.Text = "Done";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error during calibration", MessageBoxButtons.OK, MessageBoxIcon.Error);

                toolStripStatusLabel1.Text = "Error during calibration";
                status.Text = "Error";
            }
            finally
            {
                // Insure calibration is actually finished (in case of exiting on error)
                mVsa.Calibration.FieldCalibration.Cancel();

                // Reenabled GUI controls
                previewButton.Enabled = true;
                calibrateButton.Enabled = true;
                closeButton.Enabled = true;
                abortButton.Enabled = false;
            }
        }
 
        /// <summary>
        /// Set the source CW signal to the specified frequency (in Hz) and power (in dBm)
        /// </summary>
        /// <param name="frequency">desired signal frequency, in Hz</param>
        /// <param name="power">desired signal power, in dBm</param>
        private void SetSource(double frequency, double power)
        {
            // Display source frequency/power
            frequency_MHz.Text = string.Format("{0} MHz", 1.0e-6*frequency);
            sourcePower_dBm.Text = string.Format("{0:0.000} dBm", power);
            // Need to call DoEvents to allow GUI to update
            Application.DoEvents();

            // Set frequency, power, and wait to settle
            mSource.Configure(frequency, power);
        }

        /// <summary>
        /// Measure the signal power at the specified frequency.  The approximate
        /// expected power is supplied for ranging.  If simulating, the measured
        /// power is derived from the expected power and frequency with noise added.
        /// </summary>
        /// <param name="frequency">signal frequency, in Hz</param>
        /// <param name="power">expected power, in dBm</param>
        /// <returns></returns>
        private double MeasurePower(double frequency, double power)
        {
            return mPowerMeter.MeasurePower(frequency, power);
        }

        /// <summary>
        /// Check all the instruments for errors.  If any, throw an exception
        /// containing the first error found.
        /// </summary>
        private void CheckForErrors()
        {            
            mSource.CheckForErrors();
            mPowerMeter.CheckForErrors();

            int errorCode = 0;
            string errorMessage = "";
            mVsa.Utility.ErrorQuery(ref errorCode, ref errorMessage);
            if (errorCode != 0)
            {
                throw new Exception(String.Format("VSA error: {0},{1}", errorCode, errorMessage));
            }
        }
    }
}