﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace CS_FieldCalibration
{
    /// <summary>
    /// PowerMeter uses SCPI to control a variety of power meters using a VISA session.
    /// 
    /// If Simulated is set to true, it ignores the VISA resource and simply returns a
    /// power without any error.
    /// 
    /// If Simulated is set to false, it checks the identify of the source using *IDN?
    /// and if it is a recognized instrument, uses the SCPI for that instrument.  If
    /// the instrument is unrecognized, a prompt will display asking which instrument
    /// to emulate (i.e. the user selects what SCPI will be sent) -- this allows the
    /// support for a wider range of instruments that "just happen" to recognize the
    /// SCPI for one of the recognized instruments.
    /// </summary>
    public class PowerMeter : VisaSession
    {
        private const int SCPI_MODEL_UNKNOWN = 0;
        private const int SCPI_MODEL_VSERIES = 1;
        private const int SCPI_MODEL_N191X = 2;

        private const int MAXIMUM_POWER_ITERATIONS = 250;
        private const double FLATNESS_EPSILON = 0.01; // dB
        private const double BAD_CONNECTION_LIMIT = 7.5; // dB

        private readonly Random mRandomSequence = new Random(); // for simulation

        #region SCPI definitions

        // Definitions are updated in CheckModel()

        // SCPI generated with: mCalAndZeroFormat
        private string mCalAndZeroFormat;

        // SCPI generated with: String.Format(mFrequencyFormat, mFrequencyScale*frequency);
        private string mFrequencyFormat;
        private double mFrequencyScale;

        // SCPI generated with: String.Format(mPowerFormat, mPowerScale*expectedPower);
        private string mPowerFormat;
        private double mPowerScale;

        // Local delay to insure the setup is stable...
        private int mSetupDelayMs;

        // Local delay to insure instrument has sufficient time between measurement requests
        private double mMeasureDelayUs;

        // SCPI generated with:  mMeasureFormat
        private string mMeasureFormat;

        // SCPI generated with:  mOperationComplete
        private string mOperationComplete;

        // SCPI generated with:  String.Format(mMarkerFormat, mMarkerScale*frequency);
        private string mMarkerFormat;
        private double mMarkerScale;

        // SCPI generated with:  mResultFormat
        private string mResultFormat;
        private double mResultScale;
        
        #endregion

        /// <summary>
        /// Simulated indicates if Source is connected to real hardware or not (Auto-Property)
        /// </summary>
        public bool Simulated { get; set; }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="useSimulated">true to simulate source, false to use hardware</param>
        /// <param name="resource">VISA resource - ignored if useSimulated is true</param>
        public void Open(bool useSimulated, string resource)
        {
            try
            {
                Simulated = useSimulated;

                // If simulated, nothing else to do
                if (Simulated)
                {
                    return;
                }

                // Open VISA session
                OpenSession(resource);

                // Determine what SCPI model we'll use -- this will also put the
                // instrument is the correct initial state (i.e. measure CW power)
                CheckModel();
            }
            catch (Exception)
            {
                // Make sure the session is closed
                CloseSession();

                // Re-throw the exception...
                throw;
            }
        }

        /// <summary>
        /// Close the VISA session
        /// </summary>
        public void Close()
        {
            // If simulated, nothing else to do
            if (Simulated)
            {
                return;
            }

            // Close VISA session
            CloseSession();
        }

        /// <summary>
        /// Perform a power zero/calibration on the power meter.
        /// </summary>
        public void Calibrate()
        {
            // If simulated or no defined SCPI, nothing else to do
            if (Simulated || String.IsNullOrEmpty(mCalAndZeroFormat))
            {
                return;
            }

            // Execute cal and *OPC?
            mSession.WriteString(mCalAndZeroFormat);

            if (!String.IsNullOrEmpty(mOperationComplete))
            {
                // Optionally wait for completion handshake
                mSession.WriteString(mOperationComplete);
                mSession.ReadString(1000);
            }
        }

        /// <summary>
        /// Measure the signal power at the specified frequency.  The approximate
        /// expected power is supplied for ranging.  If simulating, the measured
        /// power is derived from the expected power and frequency with noise added.
        /// </summary>
        /// <param name="frequency">signal frequency, in Hz</param>
        /// <param name="expectedPower">expected power, in dBm</param>
        /// <returns></returns>
        public double MeasurePower(double frequency, double expectedPower)
        {
            string scpi;

            if (Simulated)
            {
                // Simulate loss proportional to frequency and a little noise
                return expectedPower - 0.3e-9 * frequency + 0.01 * mRandomSequence.NextDouble();
            }

            if (IsSessionOpen())
            {
                // Format and send SCPI
                if (!String.IsNullOrEmpty(mFrequencyFormat))
                {
                    // Optionally tell the device the expected frequency
                    scpi = String.Format(mFixedCulture, mFrequencyFormat, mFrequencyScale*frequency);
                    mSession.WriteString(scpi);
                }

                if (!String.IsNullOrEmpty(mPowerFormat))
                {
                    // Optionally tell the device the expected power
                    scpi = String.Format(mFixedCulture, mPowerFormat, mPowerScale * expectedPower);
                    mSession.WriteString(scpi);
                }

                // Setup delay... these gives the system time to settle from the
                // transients introduced by changing frequency/power.  The transients
                // can be non-linear, so the simple settling algorithm below does
                // not work correctly unless we delay past these transients. An
                // example transient shape using a thermocouple power sensor (8485A)
                // looks like (power vs. time):
                //
                //                                 xxxxxxxxxxxxxxxxxxxxxx
                //                         xxxxxxxx
                //   x                xxxxx
                //    xx          xxxx
                //      xxx    xxx
                //         xxxx
                //
                // The intent of the delay is to move into the portion of the
                // measurement that is monatonically increasing (or decreasing)
                // which the settling algorithm below handles.
                Thread.Sleep(mSetupDelayMs);

                // Power meters with thermocouple sensors have a non-negligible
                // thermal mass that results in a significant settling time. To
                // automatically determine the correct settling time, make
                // repeated measurements until the last 3 points shows an
                // inflection point (namely, the value is done slewing toward
                // the actual value) OR all three values are "flat" (i.e. within
                // FLATNESS_EPSILON=0.01 dB of each other).  The various cases
                // for the last 3 points:
                //   values[0] < values[1] < values[2] ... still settling
                //   values[0] < values[1] > values[2] ... settled
                //   values[0] > values[1] > values[2] ... still settling
                //   values[0] > values[1] < values[2] ... settled
                //   values[0] ~ values[1] ~ values[2] ... settled (within FLATNESS_EPSILON)
                int tries = 0;
                int maxTries = 5*MAXIMUM_POWER_ITERATIONS; // allow for transient for first point
                List<double> values = new List<double>();
                List<double> history = new List<double>();
                values.Add(0);
                values.Add(GetNextPower(frequency, ref history));
                values.Add(GetNextPower(frequency, ref history));
                do
                {
                    // Could be a length loop ... service GUI
                    Application.DoEvents();

                    // Check if maximum # iterations exceeded
                    tries += 1;
                    if (tries >= maxTries)
                    {
                        throw new Exception(String.Format("Power measurement at {0} GHz unstable after {1} attempts. Is Power Meter warmed up?", frequency * 1.0e-9, tries));
                    }
                    maxTries = MAXIMUM_POWER_ITERATIONS; // 2nd ... nth iteration limit

                    // Remove oldest power
                    values.RemoveAt(0);
                    // Get newest power
                    double value = GetNextPower(frequency, ref history);
                    values.Add(value);

                    // Sanity check
                    if (value < expectedPower - BAD_CONNECTION_LIMIT)
                    {
                        throw new Exception(String.Format("Unreasonably low power ({0:0.000} dBm). Is a cable disconnected?", value));
                    }

                    // Check for inflection point or sufficiently flat or duplicate data
                } while (
                    (values[0] < values[1] && values[1] < values[2]) ||
                    (values[0] > values[1] && values[1] > values[2]) ||
                    Math.Abs(values[0] - values[1]) > FLATNESS_EPSILON ||
                    Math.Abs(values[1] - values[2]) > FLATNESS_EPSILON ||
                    Math.Abs(values[0] - values[2]) > FLATNESS_EPSILON ||
                    values[0] == values[1] || // implies mMeasureDelayUs is too small
                    values[1] == values[2] // implies mMeasureDelayUs is too small
                    );

                // And the answer is...
                double result = (values[0] + values[1] + values[2]) / 3;

                // For debugging... compare the result with dynamic iterations to the result
                // waiting for MAXIMUM_POWER_ITERATIONS
                //double debugValue = result;
                //int debugTries = tries;
                //do
                //{
                //    Application.DoEvents();
                //    debugValue = GetNextPower(frequency, ref history);
                //    debugTries += 1;
                //} while (debugTries < MAXIMUM_POWER_ITERATIONS);
                //Console.WriteLine( String.Format("Measured {1:0.000} dBm using {0} iterations. {2:0.000} dBm at MAXIMUM_POWER_ITERATIONS", tries, result, debugValue));
                //if (Math.Abs(debugValue - result) > 0.01)
                //{
                //    Console.WriteLine( "Persisting history" );
                //    Persist(ref history);
                //}
                
                return result;
            }
            
            throw new Exception("No connection to PowerMeter");
        }

        /// <summary>
        /// For debugging -- write a list of values to a file.  Format
        /// is CSV ... appropriate for importing into Excel
        /// </summary>
        /// <param name="history"></param>
        private void Persist(ref List<double> history)
        {
            TextWriter writer = new StreamWriter("history.txt", true);
            foreach (var value in history)
            {
                writer.Write( String.Format("{0:0.000},", value));
            }
            writer.WriteLine("");
            writer.Close();
        }

        /// <summary>
        /// Get a power value from the power meter.
        /// </summary>
        /// <param name="frequency"></param>
        /// <returns></returns>
        private double GetNextPower(double frequency, ref List<double> history)
        {
            if (!String.IsNullOrEmpty(mMeasureFormat))
            {
                // If in single/triggered mode, invoke the measurement
                mSession.WriteString(mMeasureFormat);
            }

            // for free run measurements, need to make sure the instrument has
            // had time to make another measurement
            SpinWait(mMeasureDelayUs);

            if (!String.IsNullOrEmpty(mOperationComplete))
            {
                // Optionally wait for completion handshake
                mSession.WriteString(mOperationComplete);
                mSession.ReadString(1000);
            }

            if (!String.IsNullOrEmpty(mMarkerFormat))
            {
                // Optionally tell the device the expected measurement frequency
                string scpi = String.Format(mFixedCulture, mMarkerFormat, mMarkerScale * frequency);
                mSession.WriteString(scpi);
            }

            // Finally, request and read the result
            mSession.WriteString(mResultFormat);
            string result = mSession.ReadString(1000);

            double value = mResultScale * Double.Parse(result, mFixedCulture);            
            if (history != null)
            {
                history.Add(value);
            }
            return value;
        }

        /// <summary>
        /// Use the high performance counters (via Stopwatch) to implement
        /// a spin-loop wait.  This method should only be used for delays of
        /// a few milliseconds or less.
        /// </summary>
        /// <param name="delayUs"></param>
        private void SpinWait(double delayUs)
        {
            if (delayUs > 0)
            {
                double minTicks = Stopwatch.Frequency*delayUs/1.0e6;
                Stopwatch timer = new Stopwatch();
                timer.Start();
                while (timer.ElapsedTicks < minTicks)
                {
                    Thread.Sleep(0);
                }
                timer.Stop();
            }

        }

        /// <summary>
        /// Check the instrument for errors.  If there are any errors,
        /// throw the first error as an exception.
        /// </summary>
        public void CheckForErrors()
        {
            if (Simulated)
            {
                return;
            }

            mSession.WriteString(":SYST:ERR?\n");
            string response = mSession.ReadString(1000);
            char[] separators = { ',' };
            string[] fields = response.Split(separators);
            if (Int32.Parse(fields[0]) != 0)
            {
                throw new Exception("Power Meter error: " + response);
            }
        }

        /// <summary>
        /// Use the *IDN? query to determine what the connected instrument model
        /// is.  If it is one of the supported instruments, use it to define the
        /// SCPI commands.  If it is not, ask the user to select one of the supported
        /// instruments as a "SCPI model" to use and define the SCPI commands based
        /// on the selected instrument model.
        /// </summary>
        private void CheckModel()
        {
            // Ask instrument for identity
            mSession.WriteString("*IDN?\n");
            string idn = mSession.ReadString(1000);

            // Parse *IDN? response
            char[] separators = { ',' };
            string[] fields = idn.Split(separators);
            if (fields.Length != 4)
            {
                throw new Exception("Unexpected response to *IDN? - " + idn);
            }
            string manufacturer = fields[0];
            string model = fields[1];

            // Supported?
            int scpiModel = SCPI_MODEL_UNKNOWN;
            if (manufacturer.ToUpper().Contains("AGILENT"))
            {
                // For development purposes - recognize V-Series
                if (model == "MODEL 2810" ||
                    model == "MODEL 2820" ||
                    model == "MODEL 2820A" ||
                    model == "V2820A")
                {
                    scpiModel = SCPI_MODEL_VSERIES;
                }
                else if (model == "N9313A" ||
                    model == "N9314A")
                {
                    scpiModel = SCPI_MODEL_N191X;                    
                }
            }
            else if (manufacturer.ToUpper().Contains("HEWLETT-PACKARD"))
            {
                if (model == "E4418B")
                {
                    scpiModel = SCPI_MODEL_N191X;                    
                }
            }

            if (scpiModel == SCPI_MODEL_UNKNOWN)
            {
                // Ask user if one of the supported instruments is compatible
                // with the connected instrument...
                string connected = String.Format("{0} {1}", manufacturer, model);
                string[] compatibleList = {"N9313A", "N9314A"};
                ChooseModel dialog = new ChooseModel(connected, compatibleList);
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    // Convert result to enumeration
                    string compatible = dialog.CompatibleInstrument;
                    if (compatible == "N9313A" ||
                        compatible == "N9314A")
                    {
                        scpiModel = SCPI_MODEL_N191X;
                    }
                }
            }

            // Setup instruement to measure CW
            switch (scpiModel)
            {
                case SCPI_MODEL_VSERIES:
                    // Define SCPI
                    mCalAndZeroFormat = null;
                    mFrequencyFormat = ":FREQ:CENT {0:0.0########} GHz\n";
                    mFrequencyScale = 1.0e-9;
                    mPowerFormat = ":EPOW {0:0.0}\n";
                    mPowerScale = 1.0;
                    mSetupDelayMs = 50;
                    mMeasureFormat = ":INIT:IMM\n";
                    mMeasureDelayUs = 0;
                    mOperationComplete = "*OPC?\n";
                    mMarkerFormat = ":CALC:MARK1:X {0:0.0########} GHz\n";
                    mMarkerScale = 1.0e-9;
                    mResultFormat = ":CALC:MARK1:Y?\n";
                    mResultScale = 1.0;

                    // Initialize/setup up instrument for CW power measurement
                    mSession.WriteString("*RST;*OPC?\n");
                    mSession.ReadString(1000);
                    mSession.WriteString(":INIT:CONT 0\n");
                    mSession.WriteString(":FREQ:SPAN 1 MHz\n");
                    mSession.WriteString(":CALC:MARK1:STATE 1\n");
                    mSession.WriteString("*CLS;*OPC?\n");
                    mSession.ReadString(1000);
                    break;

                case SCPI_MODEL_N191X:
                    // Define SCPI
                    mCalAndZeroFormat = "CAL\n";
                    mFrequencyFormat = "FREQ {0:0.0########} GHz\n";
                    mFrequencyScale = 1.0e-9;
                    mPowerFormat = null;
                    mPowerScale = 1.0;
                    // mSetupDelayMs is empirically determined to make sure the transients from
                    // changing frequency/power have passed -- 500 ms is too small.
                    mSetupDelayMs = 750;
                    mMeasureFormat = null; // "INIT\n"; // only use INIT if :INIT:CONT OFF
                    mMeasureDelayUs = 5000;
                    mOperationComplete = null; //"*OPC?\n"; // only use *OPC? if :INIT:CONT OFF
                    mMarkerFormat = null;
                    mMarkerScale = 1.0e-9;
                    mResultFormat = "FETC?\n";
                    mResultScale = 1.0;

                    // Initialize/setup up instrument for CW power measurement
                    mSession.WriteString("SYST:PRES\n");
                    // TODO: when to run a cal?
                    //mSession.WriteString("CAL\n");
                    //mSession.ReadString(1000);
                    mSession.WriteString(":INIT:CONT ON\n");
                    mSession.WriteString("*CLS;*OPC?\n");
                    mSession.ReadString(1000);
                    break;

                default:
                    throw new Exception("Unsupported instrument: " + idn);
            }
        }
    }
}
