﻿using System;
using System.Windows.Forms;

namespace CS_FieldCalibration
{
    /// <summary>
    /// Source uses SCPI to control a variety of signal sources using a VISA session.
    /// 
    /// If Simulated is set to true, it ignores the VISA resource and simply "swallows"
    /// requests to set frequency and power without any error.
    /// 
    /// If Simulated is set to false, it checks the identify of the source using *IDN?
    /// and if it is a recognized instrument, uses the SCPI for that instrument.  If
    /// the instrument is unrecognized, a prompt will display asking which instrument
    /// to emulate (i.e. the user selects what SCPI will be sent) -- this allows the
    /// support for a wider range of instruments that "just happen" to recognize the
    /// SCPI for one of the recognized instruments.
    /// </summary>
    public class Source : VisaSession
    {
        /// <summary>
        /// Cached state of the signal output enable
        /// </summary>
        private bool mOutputEnabled;

        private const int SCPI_MODEL_UNKNOWN = 0;
        private const int SCPI_MODEL_PSG = 1;       // Agilent E8257D, E8267D, ...

        #region SCPI definitions 

        // Definitions are updated in CheckModel()

        // SCPI generated with:  String.Format(mOutputEnableFormat, enabled ? mOutputEnableStateOn : mOutputEnableStateOff);
        private string mOutputEnableFormat;
        private string mOutputEnableStateOn;
        private string mOutputEnableStateOff;

        // SCPI generated with:  String.Format(mFrequencyFormat, mFrequencyScale * frequency);
        private string mFrequencyFormat;
        private double mFrequencyScale;

        // SCPI generated with:  String.Format(mPowerFormat, mPowerScale * power);
        private string mPowerFormat;
        private double mPowerScale;

        // SCPI generated with:   mSettled  (expects a query response)
        private string mSettled;
        
        #endregion

        /// <summary>
        /// Simulated indicates if Source is connected to real hardware or not (Auto-Property)
        /// </summary>
        public bool Simulated { get; set; }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="useSimulated">true to simulate source, false to use hardware</param>
        /// <param name="resource">VISA resource - ignored if useSimulated is true</param>
        public void Open(bool useSimulated, string resource)
        {
            try
            {
                Simulated = useSimulated;

                // If simulated, nothing else to do
                if (Simulated)
                {
                    return;
                }

                // Open VISA session
                OpenSession(resource);

                // Determine what SCPI model we'll use -- this will also put the
                // instrument is the correct initial state (i.e. CW signal generation)
                CheckModel();
            }
            catch (Exception)
            {
                // Make sure the session is closed
                CloseSession();
                
                // Re-throw the exception...
                throw;
            }
        }

        /// <summary>
        /// Turn off the signal output and close the VISA session
        /// </summary>
        public void Close()
        {
            // If simulated, nothing else to do
            if (Simulated)
            {
                return;
            }

            // Make sure the output is off
            SetOutput(false);

            // Close VISA session
            CloseSession();
        }

        /// <summary>
        /// Set the source to the desired frequency and power then wait
        /// until the source is settled.
        /// </summary>
        /// <param name="frequency"></param>
        /// <param name="power"></param>
        public void Configure(double frequency, double power)
        {
            SetFrequency(frequency);
            SetPower(power);
            SetOutput(true);
            WaitForSettled();
        }

        /// <summary>
        /// Enable or disable the signal output.
        /// </summary>
        /// <param name="enabled"></param>
        public void SetOutput(bool enabled)
        {
            // If simulated, nothing else to do
            if (Simulated)
            {
                return;
            }

            if (IsSessionOpen())
            {
                // If true, only re-send SCPI if cached/previous value was false
                if (enabled && mOutputEnabled)
                {
                    return;
                }

                // Cache value
                mOutputEnabled = enabled;

                // Format and send SCPI
                string scpi = String.Format(mOutputEnableFormat, enabled ? mOutputEnableStateOn : mOutputEnableStateOff);
                mSession.WriteString(scpi);
            }
        }

        /// <summary>
        /// Set the frequency of the signal
        /// </summary>
        /// <param name="frequency">frequency, in hertz</param>
        public void SetFrequency(double frequency)
        {
            // If simulated, nothing else to do
            if (Simulated)
            {
                return;
            }

            if (IsSessionOpen())
            {
                // Format and send SCPI
                string scpi = String.Format(mFixedCulture, mFrequencyFormat, mFrequencyScale*frequency);
                mSession.WriteString(scpi);
            }
        }

        /// <summary>
        /// Set the power of the signal
        /// </summary>
        /// <param name="power">power, in dBm</param>
        public void SetPower(double power)
        {
            // If simulated, nothing else to do
            if (Simulated)
            {
                return;
            }

            if (IsSessionOpen())
            {
                // Format and send SCPI
                string scpi = String.Format(mFixedCulture, mPowerFormat, mPowerScale*power);
                mSession.WriteString(scpi);
            }
        }

        /// <summary>
        /// Wait until the source output is stable.
        /// </summary>
        public void WaitForSettled()
        {
            if (Simulated)
            {
                // If simulated, simply delay a bit to simulate real hardware
                System.Threading.Thread.Sleep(10);
                return;
            }

            if (IsSessionOpen())
            {
                // Send SCPI -- this should generate a response for handshaking
                mSession.WriteString(mSettled);
                mSession.ReadString(1000);
            }
        }

        /// <summary>
        /// Check the instrument for errors.  If there are any errors,
        /// throw the first error as an exception.
        /// </summary>
        public void CheckForErrors()
        {
            if (Simulated)
            {
                return;
            }

            mSession.WriteString(":SYST:ERR?\n");
            string response = mSession.ReadString(1000);
            char[] separators = {','};
            string[] fields = response.Split(separators);
            if (Int32.Parse(fields[0]) != 0)
            {
                throw new Exception( "Source error: " + response);
            }
        }

        /// <summary>
        /// Use the *IDN? query to determine what the connected instrument model
        /// is.  If it is one of the supported instruments, use it to define the
        /// SCPI commands.  If it is not, ask the user to select one of the supported
        /// instruments as a "SCPI model" to use and define the SCPI commands based
        /// on the selected instrument model.
        /// </summary>
        private void CheckModel()
        {
            // Ask instrument for identity
            mSession.WriteString("*IDN?\n");
            string idn = mSession.ReadString(1000);

            // Parse *IDN? response
            char[] separators = {','};
            string[] fields = idn.Split(separators);
            if (fields.Length != 4)
            {
                throw new Exception("Unexpected response to *IDN? - " + idn);
            }
            string manufacturer = fields[0].Trim();
            string model = fields[1].Trim();

            // Supported?
            int scpiModel = SCPI_MODEL_UNKNOWN;
            if (manufacturer.ToUpper().Contains("AGILENT"))
            {
                if (model == "E8257D" ||
                    model == "E8267D")
                {
                    scpiModel = SCPI_MODEL_PSG;
                }
                
            }

            if (scpiModel == SCPI_MODEL_UNKNOWN)
            {
                // Ask user if one of the supported instruments is compatible
                // with the connected instrument...
                string connected = String.Format("{0} {1}", manufacturer, model);
                string[] compatibleList = { "E8257D", "E8267D" };
                ChooseModel dialog = new ChooseModel(connected, compatibleList);
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    // Convert result to enumeration
                    string compatible = dialog.CompatibleInstrument;
                    if (compatible == "E8257D" ||
                        compatible == "E8267D")
                    {
                        scpiModel = SCPI_MODEL_PSG;
                    }
                }
            }

            // Setup instruement to generate CW
            switch (scpiModel)
            {
                case SCPI_MODEL_PSG:
                    // Define SCPI
                    mOutputEnableFormat = ":OUTP {0}\n";
                    mOutputEnableStateOn = "1";
                    mOutputEnableStateOff = "0";
                    mFrequencyFormat = ":FREQ {0:0.0########} GHz\n";
                    mFrequencyScale = 1.0e-9;
                    mPowerFormat = ":POW {0:0.0#}\n";
                    mPowerScale = 1.0;
                    mSettled = "*OPC?\n";

                    // Initialize/setup up instrument for CW signal generation
                    mSession.WriteString("*RST;*CLS;*OPC?\n");
                    mSession.ReadString(1000);
                    break;

                default:
                    throw new Exception("Unsupported instrument: " + idn);
            }
        }
    }
}
