﻿using System;
using AgM9392Lib;

namespace CS_HintUsage
{
	internal class Program
	{
		private static readonly AgM9392 vsa = new AgM9392();
		private static bool preselector;
		private static bool rfDc;
		private static bool uWaveDc;

		private static void Main( string[] args )
		{
            /*
            * You can change the code below to hard-code your resource descriptor and simulation mode, or you can pass the resource
            * descriptor in the command line when you run the example program to use that hardware in non-simulation mode.
            *
            * The resource descriptor is the same value as the “Selected Instrument” field of the SFP’s (Soft Front Panel)
            * connection dialog.   This can be a comma or semicolon separated list of module addresses such as
            *     PXI10::15::0::INSTR;PXI10::16::0::INSTR;PXI10::17::0::INSTR;PXI10::18::0::INSTR
            * or the name of a saved “Instrument Connection” created by the SFP such as
            *     PreselectedVSA
            * Refer to the SFP Help topic “Connect to Instruments and Modules” for more details.
            */

            // Pass in a command line argument as the resource descriptor, if none, will default
            string resource = "PXI20::13::INSTR";
            string options = "QueryInstrStatus=true, Simulate=true, DriverSetup= Model=, Trace=false";
            if (args.Length > 0)
            {
                resource = args[0];
                options = "QueryInstrStatus=true, Simulate=false, DriverSetup= Model=, Trace=false";
            }

			// Make a connection to the M9392 VSA interface
			vsa.Initialize( resource, true, true, options );

			try
			{
				// Determine what hardware we have available to us to work with.
				DetermineHardware();

				// If you don't have a trigger line hooked up to the M9202A Digitizer, then change this to Free Run.
				ConfigureTriggering( AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal );

				DemonstrateAttenuationHint();

				DemonstratePathPreferenceHints();
			}
			catch( Exception ex )
			{
				Console.WriteLine( "Caught an exception during execution\n{0}\nat:\n{1}", ex.Message, ex.StackTrace );
			}
			finally
			{
				vsa.Close();
                Console.WriteLine("");
                Console.WriteLine("\nDone - Press Enter to Exit");
                Console.ReadLine();
            }
		}

		/// <summary>
		/// The RF and uWave paths through the M9392 VSA driver overlap between 2.75 and 2.90 GHz due to capabilities
		/// of each of the downconverters.  The M9392 VSA driver will determine the best path to take based on frequency
		/// and bandwidth at the time of a Measurements.Initiate() command.  Between 2.75 and 2.90 GHz both paths
		/// forming "receivers" are valid and without any hint, the M9392 will bias towards using the uWave receiver.
		/// Because of the characteristics of the two downconverters it may be desirable during execution of Initiate()
		/// to "prefer" that the M9392 pick a different path.  There are two hints "Prefer RF Downconverter" and 
		/// "Prefer UWave Downconverter" that can be used to influence the selection of the path in overlapping
		/// bands.
		/// </summary>
		private static void DemonstratePathPreferenceHints()
		{
			// This only makes sense if the system as both a RF and uWave downconverter
			if( !rfDc || !uWaveDc )
			{
				Console.WriteLine(
					"Skipping DemonstratePathPreferenceHints because the M9392 VSA has only a rf or uWave Downconverter" );
				return;
			}

			vsa.Status.Clear();
			vsa.Acquisition.Hints.Clear();

			// Make a measurement at 2.8 GHZ @ -10 dBm, 40 MHz BW, with a 1 us acquisition time
			vsa.Acquisition.PreselectorEnabled = false;
			vsa.Acquisition.Frequency = 2.8e9;
			vsa.Acquisition.Power = -10;
			vsa.Acquisition.Bandwidth = 40e6;
			vsa.Acquisition.Time = 0.0001;

			// First time, force the VSA to pick the RF path

			vsa.Acquisition.Hints.Add( AgM9392HintTypeEnum.AgM9392HintTypePreferRfDownConverter, 0 );
			// 2nd Parameter is meaningless...

			vsa.Measurements.Initiate();
			vsa.Measurements.WaitForInitiateComplete( 1000 );
			vsa.Measurements.Arm();

			double power = vsa.Measurements.PeakPower;

			// Second time, force the VSA to pick the uWave path.
			vsa.Acquisition.Hints.Clear();
			vsa.Acquisition.Hints.Add( AgM9392HintTypeEnum.AgM9392HintTypePreferUWaveDownConverter, 0 );
			// 2nd Parameter is meaningless...

			vsa.Measurements.Initiate();
			vsa.Measurements.WaitForInitiateComplete( 1000 );
			vsa.Measurements.Arm();

			double powerAfter = vsa.Measurements.PeakPower;

			double deltaPower = power - powerAfter;

			Console.WriteLine( "RF Path and uWave Path yielded difference in peak power is: {0}", deltaPower );
		}

		/// <summary>
		/// Demonstrate the use of the "Force Front End Attenuation" hint.
		/// Under normal usage the M9392 VSA driver will take the input power set by the client application
		/// and determine the correct attenuation setting for the front-end preselector, however, there may be
		/// circumstances that the client would want to force the front end attenuator to some other setting.
		/// The M9360A preselector's step attenuator works in increments of 10 dB, with valid values of [0 dB, 70 dB].
		/// </summary>
		private static void DemonstrateAttenuationHint()
		{
			// if we don't have a preselector, this example is meaningless
			if( !preselector )
			{
				Console.WriteLine(
					"Skipping DemonstrateAttenuationHint because the M9392 VSA doesn't have a M9360A Preselector" );
				return;
			}

			// Show the front end attenuation hint using the RF Downconverter if there is one.
			if( rfDc )
			{
				vsa.Status.Clear();
				vsa.Acquisition.Hints.Clear();

				// Make a measurement at 2.4 GHZ @ -10 dBm, 40 MHz BW, with a 1 us acquisition time
				vsa.Acquisition.PreselectorEnabled = false;
				vsa.Acquisition.Frequency = 2.4e9;
				vsa.Acquisition.Power = -10;
				vsa.Acquisition.Bandwidth = 40e6;
				vsa.Acquisition.Time = 0.0001;

				vsa.Measurements.Initiate();
				vsa.Measurements.WaitForInitiateComplete( 1000 );
				vsa.Measurements.Arm();

				double power = vsa.Measurements.RmsPower;

				float preselectorAtten = 0;
				float dcAtten1 = 0;
				float dcAtten2 = 0;

				// Get what the system set the internal attenuators to
				vsa.Measurements.GetAttenuatorSettings( ref preselectorAtten, ref dcAtten1, ref dcAtten2 );

				// Now force the front-end attenuation to 10 dB above what it automatically picked
				double newAtten = preselectorAtten + 10;
				vsa.Acquisition.Hints.Add( AgM9392HintTypeEnum.AgM9392HintTypeForceFrontEndAttenuationTo, newAtten );

				vsa.Measurements.Initiate();
				vsa.Measurements.WaitForInitiateComplete( 1000 );
				vsa.Measurements.Arm();

				if( vsa.Measurements.DidAcquisitionOverload )
				{
					Console.WriteLine( "The VSA is overloaded." );
				}

				// Get what the system set the internal attenuators to (again)
				vsa.Measurements.GetAttenuatorSettings( ref preselectorAtten, ref dcAtten1, ref dcAtten2 );

				// If the preselector now doesn't match what we forced it to, then something bad happened.
				if( newAtten != preselectorAtten )
				{
					Console.WriteLine( "Attention, the hint apparently wasn't used!" );
				}

				double powerAfter = vsa.Measurements.RmsPower;

				double deltaPower = power - powerAfter;

				Console.WriteLine( "Increased Front End Attenation to RF Path yielded difference in RMS power is: {0}", deltaPower );
			}


			// Show the front end attenuation hint using the RF Downconverter if there is one.
			if( uWaveDc )
			{
				vsa.Status.Clear();
				vsa.Acquisition.Hints.Clear();

				// Make a measurement at 5.0 GHZ @ -10 dBm, 40 MHz BW, with a 1 us acquisition time
				vsa.Acquisition.PreselectorEnabled = false;
				vsa.Acquisition.Frequency = 5.0e9;
				vsa.Acquisition.Power = -10;
				vsa.Acquisition.Bandwidth = 40e6;
				vsa.Acquisition.Time = 0.0001;

				vsa.Measurements.Initiate();
				vsa.Measurements.WaitForInitiateComplete( 1000 );
				vsa.Measurements.Arm();

				double power = vsa.Measurements.RmsPower;

				float preselectorAtten = 0;
				float dcAtten1 = 0;
				float dcAtten2 = 0;

				// Get what the system set the internal attenuators to
				vsa.Measurements.GetAttenuatorSettings( ref preselectorAtten, ref dcAtten1, ref dcAtten2 );

				// Now force the front-end attenuation to 10 dB above what it automatically picked
				double newAtten = preselectorAtten + 10;
				vsa.Acquisition.Hints.Add( AgM9392HintTypeEnum.AgM9392HintTypeForceFrontEndAttenuationTo, newAtten );

				vsa.Measurements.Initiate();
				vsa.Measurements.WaitForInitiateComplete( 1000 );
				vsa.Measurements.Arm();

				if( vsa.Measurements.DidAcquisitionOverload )
				{
					Console.WriteLine( "The VSA is overloaded." );
				}

				// Get what the system set the internal attenuators to (again)
				vsa.Measurements.GetAttenuatorSettings( ref preselectorAtten, ref dcAtten1, ref dcAtten2 );

				// If the preselector now doesn't match what we forced it to, then something bad happened.
				if( newAtten != preselectorAtten )
				{
					Console.WriteLine( "Attention, the hint apparently wasn't used!" );
				}

				double powerAfter = vsa.Measurements.RmsPower;

				double deltaPower = power - powerAfter;

				Console.WriteLine( "Increased Front End Attenation to uWave Path yielded Difference in RMS power is: {0}",
				                   deltaPower );
			}
		}

		private static void DetermineHardware()
		{
			for( int i = 0; i < vsa.System.ModuleCount - 1; i++ )
			{
				string manufacturer = string.Empty, model = string.Empty, fwRev = string.Empty, serial = string.Empty;
				vsa.System.GetModuleIdentity( i, ref manufacturer, ref model, ref fwRev, ref serial );
				switch( model )
				{
					case "M9360A":
					case "PXI-1410":
						preselector = true;
						break;
					case "M9351A":
						rfDc = true;
						break;
					case "M9361A":
						uWaveDc = true;
						break;
				}
			}
		}

		private static void ConfigureTriggering( AgM9392TriggerSourceEnum trigger )
		{
			switch( trigger )
			{
				case AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal:
					vsa.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
					vsa.Trigger.Delay = 0;
					vsa.Trigger.External.Configure( 1, AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive );
					break;
				case AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate:
					vsa.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate;
					break;
				case AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideo:
					Console.WriteLine( "Video triggering isn't configurable in this example" );
					break;
			}
		}
	}
}