// One of the challenges of programming a receiver for an automated test is dealing
// with situations where the test device does not provide the expected signal.  
// Perhaps it is broken, perhaps it is at the wrong frequency.  If the receiver 
// is expecting a rising edge to trigger on, you can end up waiting a long time 
// for the system to figure out that there is no signal there.  The M9392 uWave 
// VSA is designed to be automated test friendly  you can tell if it has been 
// triggered or not, and you can set timeouts at every step of the acquisition.  
//
// For example, if the test device is supposed to be providing a signal with a 
// pulses every 1.5 ms, and you were careful not to Arm the receiver until the 
// test device should have been generating a signal, you could set a trigger 
// timeout of 5 ms.   You can query whether a timeout has occurred before reading 
// the data, so you dont waste time reading data the doesnt mean anything.
//
// Alternatively, there are query methods so that you can determine if the 
// acquisition system has received a trigger, and whether the acquisition has 
// completed and data is available.  This is achieved with WaitForTrigger() and 
// WaitForData(), passing in zero for the timeout to create a non-blocking query.
//
// In this example, we assume a nominal 1.5 ms pulse rep rate, with a pulse-on 
// power of -10 dBm, and a trigger level of -20 dBm.  We will use a 5 ms timeout 
// to avoid waiting for a bad signal, and check for the timeout.  The signal is 
// at 10.5 GHz, and is 10 MHz wide.

using System;
using System.Windows.Forms;
using AgM9392Lib;

namespace CS_VideoTrigger
{
    public partial class VideoTrigger : Form
    {
        private AgM9392 mVsa;
        private readonly double PRESELECTOR_MIN_FREQ = 2.9e9;

        public VideoTrigger()
        {
            InitializeComponent();

            toolStripStatusLabel1.Text = "Ready (Open the VSA next)";
        }

        private void openButton_Click(object sender, EventArgs e)
        {
            toolStripStatusLabel1.Text = "Opening VSA...";

            try
            {
                string options = Options.Text;
                if (simulatedCheck.Checked)
                {
                    options = "Simulate=TRUE," + options;
                }

                // Instantiate and Initialize the M9392A VSA driver 
                mVsa = new AgM9392Class();
                mVsa.Initialize(IOResourceDescriptor.Text, false, false, options);

                // adjust GUI enables...
                simulatedCheck.Enabled = false;
                openButton.Enabled = false;
                closeButton.Enabled = true;
                initiateButton.Enabled = true;

                // The instrument will perform power on self test as it
                // connects to the modules and verifies the configuration.
                // Technically, it will already be in the preset state when
                // the constructor call returns.  For good form, we
                // force the hardware to a known state again.
                mVsa.Utility.Reset();

                toolStripStatusLabel1.Text = "Ready";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error opening VSA", MessageBoxButtons.OK, MessageBoxIcon.Error);

                // Get rid of the M9392A VSA driver and adjust GUI enables...
                mVsa = null;

                simulatedCheck.Enabled = true;
                openButton.Enabled = true;
                closeButton.Enabled = false;
                initiateButton.Enabled = false;

                toolStripStatusLabel1.Text = "Error opening VSA";
            }
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            // Get rid of the M9392A VSA driver and adjust GUI enables...
            mVsa = null;

            simulatedCheck.Enabled = true;
            openButton.Enabled = true;
            closeButton.Enabled = false;
            initiateButton.Enabled = false;

            toolStripStatusLabel1.Text = "Ready (Open the VSA next)";
            toolStripTriggerTimeout.Visible = false;
            toolStripOverload.Visible = false;
        }

        private void initiateButton_Click(object sender, EventArgs e)
        {
            try
            {
                toolStripStatusLabel1.Text = "Setting VSA parameters...";

                // Clear any existing errors
                mVsa.Status.Clear();

                // Convert settings to base units...
                double frequency = 1.0e9*double.Parse(frequency_GHz.Text);
                double power = double.Parse(power_dBm.Text);
                double bandwidth = 1.0e3*double.Parse(bandwidth_kHz.Text);
                double acquisitionTime = 1.0e-3*double.Parse(acquisitionTime_ms.Text);
                bool usePreselector = PreselectorBox.Checked && frequency >= PRESELECTOR_MIN_FREQ;

                // We can use a single call to set up the receiver with the
                // most common parameters
                mVsa.Acquisition.Configure(
                    usePreselector,
                    frequency, // in Hz
                    power, // in dBm
                    bandwidth, // in Hz
                    acquisitionTime, // in s
                    AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideo);

                // Having activated video trigger, we need to set all the 
                // video trigger parameters
                mVsa.Trigger.Delay = 0; // in s ... zero delay, we want to see edge
                mVsa.Trigger.Video.Level = -20; // in dBm
                mVsa.Trigger.Video.Slope = AgM9392VideoTriggerSlopeEnum.AgM9392VideoTriggerSlopePositive;

                // Now we set the trigger timeout, so we dont wait for a
                // signal that is not there
                mVsa.Trigger.Timeout = 5; // in ms

                // The driver now has all the information it needs to make a
                // measurement.  Nothing has been written to the hardware, and
                // the requested settings have not been checked against the
                // available hardware.  This all happens when we call the
                // Initiate routine, which selects the specific hardware path
                // to be used in the measurement, and starts the hardware
                // slewing to the new state.
                try
                {
                    toolStripStatusLabel1.Text = "Initiate measurement...";
                    mVsa.Measurements.Initiate();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                    // If an error occurred, there was a problem meeting the
                    // requested settings, and we need to read the messages in
                    // the message queue to find out what is wrong.
                    while (mVsa.Status.NumberOfErrors > 0)
                    {
                        int errorCode = 0;
                        string errorMessage = "No Error";
                        mVsa.Utility.ErrorQuery(ref errorCode, ref errorMessage);
                        MessageBox.Show(String.Format("{0} : {1}", errorCode, errorMessage),
                                        "Reported Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }

                // At this point, all the acquisition hardware is ready to
                // start looking for a video trigger.  This means the
                // LO is at frequency, all the switches have settled, and
                // the acquisition system has been configured.  The Arm()
                // command will start the search for the video trigger.
                toolStripStatusLabel1.Text = "Arming...";
                mVsa.Measurements.Arm();

                // Now that acquisition has begun, we take a moment to allocate
                // the buffer to hold the complex measured data, which is 32 bit
                // floats, interleaved real and imag. The user will probably
                // also want to know what sample rate the data was taken at  the
                // sample rate varies with the requested bandwidth.  Use
                //  GetCurrentSampleRate() to query the sample rate.
                int numSamples = mVsa.Measurements.NumberOfSamples;
                Array myData = new float[2 * numSamples];

                // Now we block until data is available.  We dont ask for
                // the data yet because we dont want the data if there
                // has been a trigger timeout.  We know this method will not wait
                // longer than 5 ms because of the trigger timeout, but it has
                // its own concept of timeout, so we pass in an arbitrary large
                // value
                toolStripStatusLabel1.Text = "Waiting for trigger...";
                AgM9392WaitStatusEnum status = mVsa.Measurements.WaitForData(2000);
                if (status == AgM9392WaitStatusEnum.AgM9392WaitStatusCompleteWithTimeout )
                {
                    // Clear displayed data
                    rmsPower_dBm.Text = "";
                    peakPower_dBm.Text = "";
                    dataList.Items.Clear();
                    throw new Exception("Video trigger did not occur, measurement auto-triggered");
                }
                if (status == AgM9392WaitStatusEnum.AgM9392WaitStatusWaitExpired )
                {
                    // Something is wrong -- should not have timed out!
                    mVsa.Measurements.Abort();
                    throw new Exception("Error: WaitForData timed out!");
                }

                // The data is valid, lets go retrieve it
                toolStripStatusLabel1.Text = "Waiting for data...";
                Int32 numSamplesReturned = 0;
                mVsa.Measurements.GetData(
                    ref myData, // the data buffer
                    ref numSamplesReturned); // # of samples actually copied to myData

                // Query the measured power of the signal
                double rmsPower = mVsa.Measurements.RmsPower;
                double peakPower = mVsa.Measurements.PeakPower;

                // Lastly, we check to see that all went well with the
                // acquisition.  Unlike a SCPI instrument, checking status is
                // very low overhead and should be done frequently.  We want
                // to check for messages in the message queue, as well as
                // specifically check for overload
                while (mVsa.Status.NumberOfErrors > 0)
                {
                    int errorCode = 0;
                    string errorMessage = "No Error";
                    mVsa.Utility.ErrorQuery(ref errorCode, ref errorMessage);
                    MessageBox.Show(String.Format("{0} : {1}", errorCode, errorMessage),
                                    "Reported Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                // Now check for overload
                toolStripOverload.Visible = mVsa.Measurements.DidAcquisitionOverload;

                // Now check if the trigger timed out
                toolStripTriggerTimeout.Visible = mVsa.Measurements.DidTriggerTimeout;

                // Display the data
                rmsPower_dBm.Text = rmsPower.ToString();
                peakPower_dBm.Text = peakPower.ToString();

                dataList.Items.Clear();
                for (int j = 0; j < numSamples; j++)
                {
                    string iq = string.Format("{0} : {1}", myData.GetValue(2 * j), myData.GetValue(2 * j + 1));
                    dataList.Items.Add(iq);
                }

                toolStripStatusLabel1.Text = "Ready";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error invoking measurement", MessageBoxButtons.OK, MessageBoxIcon.Error);

                toolStripStatusLabel1.Text = "Error invoking measurement";
            }
        }
    }
}