/******************************************************************************
*  Copyright (c) 2010 Agilent Technologies Inc.  All rights reserved.
*
* You have a royalty-free right to use, modify, reproduce and distribute
* the sample application files (and/or any modified version) in any way
* you find useful, provided that you agree that Agilent Technologies has no
* warranty, obligations or liability for any sample application files.
*
* Agilent Technologies provides programming examples for illustration only,
* This sample program assumes that you are familiar with the programming
* language being demonstrated and the tools used to create and debug
* procedures. Agilent Technologies support engineers can help explain the
* functionality of Agilent Technologies software components and associated
* commands, but they will not modify these samples to provide added
* functionality or construct procedures to meet your specific needs.
*****************************************************************************/
///
/// Agilent IVI-C Driver Example Program
/// 
/// Initializes the driver, reads a few Identity interface
/// properties, and initiates instrument specific functionality.
/// 
/// Runs in simulation mode without an instrument.
/// 

#include "stdafx.h"
#include <atlstr.h>

#import "IviDriverTypeLib.dll"	no_namespace // IVI inherent functionality

#import "AgM9392.dll"	no_namespace

bool binaryPrompt(char* label, bool def)
{
	while(true)
	{
		printf("%s?\n   (enter 'yes' or 'no'. default=%s): ", label,(def)?"yes":"no");
		char answer[1024];
		gets_s(answer,1024);
		switch(answer[0])
		{
		case 0:
			return def;
		case 'y':
		case 'Y':
			return true;
		case 'n':
		case 'N':
			return false;
		default:
			printf("Please respond 'yes' or 'no':\n");
		}
	}
}

double numberPrompt(char* label, double min, double max, double defval)
{
	double value;
	while (true)
	{
		printf("%s?\n   (enter a value between %0.3lf and %0.3lf. default=%0.3lf): ", label, min, max, defval);
		char answer[1024];
		gets_s(answer,1024);
		if(answer[0] > 0)
		{
			sscanf_s(answer,"%lf",&value);
		}
		else
		{
			value = defval;
		}
		if ((value >= min) && (value <= max)) return value;
		printf("     Response must be between %0.3lf and %0.3lf.\n", min, max);
	}            
}

void setupAcquisition(IAgM9392Ptr driver)
{
	double PRESELECTOR_MIN_FREQ = 2.9e9;
	char prompt[1024];
	sprintf_s(prompt,"Use Preselector (%0.3lf-%0.3lf GHz)", PRESELECTOR_MIN_FREQ / 1e9, driver->Acquisition->MaxFrequency / 1e9);
	driver->Acquisition->PreselectorEnabled = binaryPrompt(prompt, false);
	driver->Acquisition->Frequency = 1e9 * numberPrompt("Frequency (GHz)", 
		driver->Acquisition->MinFrequency / 1e9,
		driver->Acquisition->MaxFrequency / 1e9, 8.0);
	if((driver->Acquisition->Frequency < PRESELECTOR_MIN_FREQ) && driver->Acquisition->PreselectorEnabled)
	{
		printf("     DISABLING preselector (frequency too low)\n");
		driver->Acquisition->PreselectorEnabled = false;
	}
	driver->Acquisition->Power = numberPrompt("Power (dBm)", 
		driver->Acquisition->MinPower,
		driver->Acquisition->MaxPower, -10);
	double minBW = 1000; // Hz
	driver->Acquisition->Bandwidth = 1e6 * numberPrompt("Bandwidth (MHz)", minBW / 1e6, driver->Acquisition->MaxBandwidth / 1e6, 1);
	double maxTime = 1; // the maximum actually depends on other setup conditions, please refer to the documentation
	double minTime = 10e-3; // the minimum actually depends on other setup conditions, please refer to the documentation
	driver->Acquisition->Time = 1e-3 * numberPrompt("Acquisition Time (ms)", minTime * 1e3, maxTime * 1e3, 20);
	driver->Calibration->FieldCalibration->Enabled = binaryPrompt("Calibration", true);
}

void acquireMeasurement(IAgM9392Ptr driver, int timeout)
{
	SAFEARRAY* complexdata = NULL;
	printf("\n");
	printf("Initiate measurement...\n");
	driver->Measurements->Initiate();
	driver->Measurements->WaitForInitiateComplete(timeout);
	printf("Arming...\n");
	driver->Measurements->Arm();

	driver->Measurements->WaitForData(timeout);

	printf("RMS Power = %lf dBm.\n", driver->Measurements->RmsPower);
	printf("Peak Power = %lf dBm.\n", driver->Measurements->PeakPower);

	long nsam = driver->Measurements->NumberOfSamples;
	long numCopied = 0;
	//GetData will allocate memory
	driver->Measurements->GetData(&complexdata, &numCopied);
	printf("Acquired %d complex data samples at %g per second.\n", nsam, driver->Measurements->SampleRate);
	nsam = (long)numberPrompt("Number of samples to display", 0, (double)numCopied/2, (double)numCopied/2);

	if(nsam>0)
	{
		float* vectors = (float*)complexdata->pvData;
		printf("	#		I		Q\n");
		for (long n = 0; n < nsam; n ++)
		{
			printf("   %d      %f      %f\n",n,vectors[2*n],vectors[2*n+1]);   
		}
	}
}

int _tmain(int argc, _TCHAR* argv[])
{
	::CoInitialize(NULL);
	printf("Cpp_BasicMeasurement\n\n");

	/*
	* You can change the code below to hard-code your resource descriptor and simulation mode, or you can pass the resource
    * descriptor in the command line when you run the example program to use that hardware in non-simulation mode.
	*
    * The resource descriptor is the same value as the Selected Instrument field of the SFPs (Soft Front Panel)
    * connection dialog.   This can be a comma or semicolon separated list of module addresses such as
    *     PXI10::15::0::INSTR;PXI10::16::0::INSTR;PXI10::17::0::INSTR;PXI10::18::0::INSTR
    * or the name of a saved Instrument Connection created by the SFP such as
    *     PreselectedVSA
    * Refer to the SFP Help topic Connect to Instruments and Modules for more details.
	*/

	// Pass in a command line argument as the resource descriptor, if none, will default
	char* resource = "PXI20::13::INSTR";
	char* options = "QueryInstrStatus=true, Simulate=true, DriverSetup= Model=, Trace=false";
	if (argc > 1)
	{
		resource = argv[1];
    	options  = "QueryInstrStatus=true, Simulate=false, DriverSetup= Model=, Trace=false";
	}

	IAgM9392Ptr spDriver = NULL;
	
	try{
		spDriver.CreateInstance(__uuidof(AgM9392));

		VARIANT_BOOL idQuery = VARIANT_TRUE;
		VARIANT_BOOL reset   = VARIANT_TRUE;

		// Initialize the driver.  See driver help topic "Initializing the IVI-COM Driver" for additional information

		spDriver->Initialize(LPCTSTR(resource), idQuery, reset, LPCTSTR(options));	
		printf("Driver Initialized\n");

		// Print a few IIviDriverIdentity properties
		wprintf(L"Identifier:  %s\n", spDriver->Identity->Identifier.GetBSTR());
		wprintf(L"Revision:    %s\n", spDriver->Identity->Revision.GetBSTR());
		wprintf(L"Vendor:      %s\n", spDriver->Identity->Vendor.GetBSTR());
		wprintf(L"Description: %s\n", spDriver->Identity->Description.GetBSTR());
		wprintf(L"Model:       %s\n", spDriver->Identity->InstrumentModel.GetBSTR());
		wprintf(L"FirmwareRev: %s\n", spDriver->Identity->InstrumentFirmwareRevision.GetBSTR());
		
		wprintf(L"Serial #:    %s\n", spDriver->System->SerialNumber.GetBSTR());
		wprintf(L"Simulate:    %s\n", (spDriver->DriverOperation->Simulate==VARIANT_TRUE)?L"True":L"False");
		wprintf(L"\n");

		setupAcquisition(spDriver);
		if (binaryPrompt("Initiate", true))
		{
			acquireMeasurement(spDriver, 1000);
		}

		/*
         * Polling the instrument for queued errors is not necessary.  No errors are queued.
         * rather, each of the above driver calls would have thrown an exception if an error occurred.
         * Any thrown exceptions would be handled in the catch clause below:
		 */
	}
	catch (_com_error& e)
	{
		::MessageBox(NULL, e.Description(), e.ErrorMessage(), MB_ICONERROR);
	}

	if ((spDriver != NULL) && (spDriver->Initialized))
	{
		// Close the driver
		try
		{
			spDriver->Close();

			// It is necessary to release the driver object.
			spDriver.Release();
		}
		catch (_com_error& e)
		{
			::MessageBox(NULL, e.Description(), e.ErrorMessage(), MB_ICONERROR);
		}

		printf("Driver Closed\n");
	}

	printf("\nDone - Press Enter to Exit");

	getchar();

	::CoUninitialize();

	return 0;
}


