﻿'Field Calibration

'The M9392 Microwave VSA is constructed out of PXI modules.  Any time the front panel cables 
'are disconnected and reconnected, the power accuracy will suffer.  Similarly, if one of the
'modules is swapped out, there is going to be a small perturbation to the path loss, and
'hence the power accuracy.

'To provide best power accuracy, the M9392 should be calibrated in the field.  To perform 
'this calibration, you need a microwave source that can cover the entire 40 MHz to 26.5 GHz,
'and a NIST traceable power meter and sensor.

'The cal procedure is to first use the GetCalFreqList() query to find out the complete 
'frequency list the M9392 needs to be calibrated over.  The actual list generated will 
'depend on which modules are present in the instrument.

'The user (or the automating program) then needs to loop over that frequency list, setting 
'the source to each frequency at a nominal -10 dBm, measuring the actual power delivered 
'to the M9392, and calling back to the M9392 using the CalThisFreq() method, which causes 
'the M9392 to calibrate all paths that use that input frequency.

'There are two ways to provide the calibrated input power to the M9392.  The easiest way is 
'to connect a power meter directly to the microwave source output, and step through the 
'frequency list, measuring the source output power at each frequency.  Then the user connects 
'the microwave source to the M9392 input, and we sequence the source through the exact same 
'list (same frequencies, same powers), relying on the repeatability and leveling of the 
'source to provide the same power to the M9392 as to the power meter.

'The more accurate approach is to use a two resister power splitter and and a dual power 
'meter.  The common terminal of the splitter is connected to the source output.  One power 
'sensor is “permanently” connected to one leg of the splitter.  To calibrate the splitter, 
'we step through the frequency list with a second power sensor on the other leg of the
'splitter, recording the difference between the power meter readings as the splitter 
'calibration.  Then we disconnect the second power sensor and connect the M9392 receiver. 
'At each frequency in the list, cache the input power recorded by the “permanent” sensor, 
'apply the splitter correction, and pass the resulting power into CalThisFreq().

'It is recommended that the power level at the M9392 input is between -10 and -15 dBm.   
'If a power splitter is used, or if some lossy fixturing is being included in the 
'calibration, it is recommended that a first order correction of the source power be done
'to keep the power in the -10 to -15 dBm range.

'The customer can include parts of the test fixturing, cabling and switching in the 
'calibration.  The power at the M9392 input should be maintained between -10 and -15 dBm. 
'Notice that the frequency spacing of the calibration is relatively coarse – the calibration
'will not work well for frequency selective devices that have significant change in insertion
'loss across a 25 MHz step.

'For manual operation, the M9392 Soft Front Panel will guide the customer through the
'calibration.  The customer must have one of the following microwave sources:
'•	E8257D PSG covering 26.5 GHz (Analog/CW)
'•	E8267D PSG covering 26.5 GHz (Vector Signal Generator)
'And one of the following Power Meters
'•	N1913A EPM Single Channel Power Meter 
'•	N1914A EPM Dual Channel Power Meter
'And the following power sensor:
'•	N8485A 10 MHz – 26.5 GHz Thermocouple -35 to +20 dBm  ** recommended **

'This example below demonstrates how to do the simpler calibration, with no power splitter.

Imports AgM9392Lib
Imports VB_FieldCalibration.VB_FieldCalibration
Imports System.Threading

Partial Class FieldCalibration

    Private Const DESIRED_POWER As Double = -10
    ' desired power at VSA for calibration
    Private Const MINIMUM_POWER As Double = -12.5
    ' minimum power at VSA for calibration
    Private Const MAXIMUM_POWER As Double = 10
    ' maximum power to be requested from source
    Private mVsa As AgM9392
    Private mAbort As Boolean
    Private ReadOnly mSource As New Source
    Private ReadOnly mPowerMeter As New PowerMeter

    Public Sub New()
        InitializeComponent()
    End Sub

    Private Sub simulatedSourceCheck_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles simulatedSourceCheck.CheckedChanged
        sourceResourceDescriptor.Enabled = Not simulatedSourceCheck.Checked
    End Sub

    Private Sub simulatedMeterCheck_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles simulatedMeterCheck.CheckedChanged
        meterResourceDescriptor.Enabled = Not simulatedMeterCheck.Checked

    End Sub

    Private Sub simulatedVsaCheck_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles simulatedVsaCheck.CheckedChanged
        vsaResourceDescriptor.Enabled = Not simulatedVsaCheck.Checked
    End Sub

    Private Sub openButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles openButton.Click

        toolStripStatusLabel1.Text = "Opening VSA..."
        Cursor.Current = Cursors.WaitCursor

        Try
            Dim options As String = If(simulatedVsaCheck.Checked, "Simulate=TRUE", "")

            ' Instantiate and Initialize the M9392A VSA driver 
            mVsa = New AgM9392Class()
            mVsa.Initialize(vsaResourceDescriptor.Text, False, False, options)

            ' adjust GUI enables...
            simulatedVsaCheck.Enabled = False
            simulatedSourceCheck.Enabled = False
            simulatedMeterCheck.Enabled = False
            vsaResourceDescriptor.Enabled = False
            sourceResourceDescriptor.Enabled = False
            meterResourceDescriptor.Enabled = False
            openButton.Enabled = False
            closeButton.Enabled = True
            previewButton.Enabled = True
            calibrateButton.Enabled = True
            abortButton.Enabled = False

            ' The instrument will perform power on self test as it
            ' connects to the modules and verifies the configuration.
            ' Technically, it will already be in the preset state when
            ' the constructor call returns.  For good form, we
            ' force the hardware to a known state again.
            mVsa.Utility.Reset()

            ' Open the source
            mSource.Open(simulatedSourceCheck.Checked, sourceResourceDescriptor.Text)

            ' Open the power meter
            mPowerMeter.Open(simulatedMeterCheck.Checked, meterResourceDescriptor.Text)

            toolStripStatusLabel1.Text = "Ready"
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error opening instrument(s)", MessageBoxButtons.OK, MessageBoxIcon.[Error])

            ' Close I/O
            CloseInstruments()

            ' Update GUI
            ShowClosedIO()

            toolStripStatusLabel1.Text = "Error opening instrument(s)."
        End Try
        Cursor.Current = Cursors.[Default]
    End Sub

    Private Sub closeButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles closeButton.Click
        Cursor.Current = Cursors.WaitCursor

        ' Close I/O
        CloseInstruments()

        ' Update GUI
        ShowClosedIO()

        Cursor.Current = Cursors.[Default]
    End Sub

    Private Sub FieldCalibration_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles Me.FormClosing

        ' When the form is closing, be sure the devices are closed
        ' to release any VISA locks
        CloseInstruments()

        ' Don't worry about updating the GUI -- we're exiting!
    End Sub

    ''' <summary>
    ''' Close all instrument connections
    ''' </summary>
    Private Sub CloseInstruments()
        Try
            ' Get rid of the M9392A VSA driver and adjust GUI enables...
            mVsa = Nothing
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error while closing the VSA instrument connection", MessageBoxButtons.OK, MessageBoxIcon.[Error])
        End Try

        Try
            ' Close the source
            mSource.Close()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error while closing the Source instrument connection", MessageBoxButtons.OK, MessageBoxIcon.[Error])
        End Try

        Try
            ' Close the power meter
            mPowerMeter.Close()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error while closing the Power Meter instrument connection", MessageBoxButtons.OK, MessageBoxIcon.[Error])
        End Try
    End Sub

    ''' <summary>
    ''' Update the GUI controls to indicate that I/O is closed
    ''' </summary>
    Private Sub ShowClosedIO()
        simulatedVsaCheck.Enabled = True
        simulatedSourceCheck.Enabled = True
        simulatedMeterCheck.Enabled = True
        vsaResourceDescriptor.Enabled = Not simulatedVsaCheck.Checked
        sourceResourceDescriptor.Enabled = Not simulatedSourceCheck.Checked
        meterResourceDescriptor.Enabled = Not simulatedMeterCheck.Checked
        openButton.Enabled = True
        closeButton.Enabled = False
        previewButton.Enabled = False
        calibrateButton.Enabled = False
        abortButton.Enabled = False

        frequencyList.Items.Clear()
        status.Text = ""
        frequency_MHz.Text = ""
        sourcePower_dBm.Text = ""
        measuredPower_dBm.Text = ""

        toolStripStatusLabel1.Text = "Ready (Open the VSA next)"
        toolStripTriggerTimeout.Visible = False
        toolStripOverload.Visible = False
    End Sub

    Private Sub previewButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles previewButton.Click
        Try
            ' This method is not part of the calibration, but simply
            ' lets the user peruse the frequencies the instrument
            ' wants to be calibrated at
            Dim frequencies As Array = New Double(-1) {}
            mVsa.Calibration.FieldCalibration.GetFrequencyList(frequencies)

            frequencyList.Items.Clear()
            For Each frequency As Double In frequencies
                frequencyList.Items.Add([String].Format("{0} MHz", 0.000001 * frequency))
            Next
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error retrieving calibration frequencies", MessageBoxButtons.OK, MessageBoxIcon.[Error])

            toolStripStatusLabel1.Text = "Error retrieving calibration frequencies"
        End Try
    End Sub

    Private Sub abortButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles abortButton.Click
        ' Just set a flag that calibrateButton_Click() polls for
        mAbort = True
    End Sub

    Private Sub calibrateButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles calibrateButton.Click
        Try
            ' Disable GUI controls
            previewButton.Enabled = False
            calibrateButton.Enabled = False
            closeButton.Enabled = False
            abortButton.Enabled = True

            ' Optionally perform a cal/zero of the power meter
            toolStripStatusLabel1.Text = "Calibrate power meter"
            Dim result As DialogResult = MessageBox.Show("Connect the power meter to the power reference and click 'Yes' to calibrate the power meter." & vbLf & "Click 'No' to skip power meter calibration", "User Action Required", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question)
            If result = DialogResult.Cancel Then
                status.Text = "Calibration aborted"
                toolStripStatusLabel1.Text = "Calibration aborted"
                Return
            End If
            If result = DialogResult.Yes Then
                mPowerMeter.Calibrate()
                MessageBox.Show("Wait until the power meter has finished calibrating then click 'OK'", "User Action Required", MessageBoxButtons.OK)
                CheckForErrors()
            End If

            ' Instruct the user to connect source to power meter
            toolStripStatusLabel1.Text = "Connect power meter"
            If MessageBox.Show("Connect source to power meter", "User Action Required", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation) = DialogResult.Cancel Then
                status.Text = "Calibration aborted"
                toolStripStatusLabel1.Text = "Calibration aborted"
                Return
            End If

            ' Retrieve the list of calibration frequencies 
            Dim frequencies As Array = New Double(-1) {}
            mVsa.Calibration.FieldCalibration.GetFrequencyList(frequencies)

            ' Create the source power and measured power arrays
            Dim sourcePower As Double() = New Double(frequencies.Length - 1) {}
            Dim measuredPower As Double() = New Double(frequencies.Length - 1) {}

            ' Initially, assume no loss...
            Dim power As Double = DESIRED_POWER

            ' Especially with themocouple sensors, the first setup may take
            ' quite a while to settle since the source is initially off.  So
            ' set up the first frequency/power pair and wait 5 seconds...
            If frequencies.Length > 0 Then
                Dim frequency As Double = CDbl(frequencies.GetValue(0))
                SetSource(frequency, power)
                Thread.Sleep(5000)

                ' Now measure the initial power...
                Dim measured As Double = MeasurePower(frequency, power)

                ' Adjust the next source power to target DESIRED_POWER
                If Math.Abs(DESIRED_POWER - measured) > 0.1 Then
                    power += DESIRED_POWER - measured
                    power = Math.Min(power, MAXIMUM_POWER)
                End If

                ' Check for errors...
                CheckForErrors()
            End If

            ' Loop over the frequencies and measure the actual
            ' power at the end of the cable (where the power meter
            ' is connected)
            toolStripStatusLabel1.Text = "Measuring power"
            status.Text = "Measuring power"
            For index As Integer = 0 To frequencies.Length - 1
                Dim frequency As Double = CDbl(frequencies.GetValue(index))

                ' Set the source frequency/power
                SetSource(frequency, power)

                ' Measure the power ... want between DESIRED_POWER and MINIMUM_POWER
                Dim measured As Double = MeasurePower(frequency, power)
                If measured < MINIMUM_POWER Then
                    ' Too small!  Try one more time with correction at the source
                    power += DESIRED_POWER - measured
                    power = Math.Min(power, MAXIMUM_POWER)

                    ' Set the source frequency/power
                    SetSource(frequency, power)

                    ' Measure again
                    measured = MeasurePower(frequency, power)
                End If

                ' Display measured power
                measuredPower_dBm.Text = String.Format("{0:0.000} dBm", measured)

                ' Cache the source and measured powers
                sourcePower(index) = power
                measuredPower(index) = measured

                ' Adjust the next source power to target DESIRED_POWER
                If Math.Abs(DESIRED_POWER - measured) > 0.1 Then
                    power += DESIRED_POWER - measured
                    power = Math.Min(power, MAXIMUM_POWER)
                End If

                ' Check for errors...
                CheckForErrors()

                ' Check for abort
                If mAbort Then
                    mAbort = False
                    If MessageBox.Show("Are you sure you want to abort the calibration?", "Abort Calibration?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
                        status.Text = "Calibration aborted"
                        toolStripStatusLabel1.Text = "Calibration aborted"
                        Return
                    End If
                End If
            Next

            ' Instruct the user to connect source to M9392A
            If MessageBox.Show("Connect source to M9392A", "User Action Required", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation) = DialogResult.Cancel Then
                status.Text = "Calibration aborted"
                toolStripStatusLabel1.Text = "Calibration aborted"
                Return
            End If

            ' Loop over the frequencies (again) and tell the M9392A
            ' to calibrate at each frequency with the power measured
            ' by the power meter in the previous loop
            toolStripStatusLabel1.Text = "Performing calibration"
            status.Text = "Performing calibration"
            For index As Integer = 0 To frequencies.Length - 1
                Dim frequency As Double = CDbl(frequencies.GetValue(index))

                ' Set the source frequency/power
                SetSource(frequency, sourcePower(index))

                ' Display the previously measured power
                measuredPower_dBm.Text = String.Format("{0:0.000} dBm", measuredPower(index))

                ' Calibrate... the first time tis is executed the VSA enters
                ' "calibration mode" and will remain in this mode until Cancel or Save.
                ' Calls to Measurements.Initiate() will generate errors while in calibration mode.
                mVsa.Calibration.FieldCalibration.CalThisFrequency(frequency, measuredPower(index))

                ' Check for errors...
                Try
                    CheckForErrors()
                Catch generatedExceptionName As Exception
                    ' Need to tell VSA to exit calibration mode
                    mVsa.Calibration.FieldCalibration.Cancel()
                    Throw
                End Try

                ' Check for abort
                If mAbort Then
                    mAbort = False
                    If MessageBox.Show("Are you sure you want to abort the calibration?", "Abort Calibration?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes Then
                        status.Text = "Calibration aborted"
                        toolStripStatusLabel1.Text = "Calibration aborted"
                        ' Need to tell VSA to exit calibration mode
                        mVsa.Calibration.FieldCalibration.Cancel()
                        Return
                    End If
                End If
            Next

            ' Done!  Save the results
            toolStripStatusLabel1.Text = "Saving calibration"
            mVsa.Calibration.FieldCalibration.Save()
            toolStripStatusLabel1.Text = "Ready"
            status.Text = "Done"
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error during calibration", MessageBoxButtons.OK, MessageBoxIcon.[Error])

            toolStripStatusLabel1.Text = "Error during calibration"
            status.Text = "Error"
        Finally
            ' Reenabled GUI controls
            previewButton.Enabled = True
            calibrateButton.Enabled = True
            closeButton.Enabled = True
            abortButton.Enabled = False
        End Try
    End Sub

    ''' <summary>
    ''' Set the source CW signal to the specified frequency (in Hz) and power (in dBm)
    ''' </summary>
    ''' <param name="frequency">desired signal frequency, in Hz</param>
    ''' <param name="power">desired signal power, in dBm</param>
    Private Sub SetSource(ByVal frequency As Double, ByVal power As Double)
        ' Display source frequency/power
        frequency_MHz.Text = String.Format("{0} MHz", 0.000001 * frequency)
        sourcePower_dBm.Text = String.Format("{0:0.000} dBm", power)
        ' Need to call DoEvents to allow GUI to update
        Application.DoEvents()

        ' Set frequency, power, and wait to settle
        mSource.Configure(frequency, power)
    End Sub

    ''' <summary>
    ''' Measure the signal power at the specified frequency.  The approximate
    ''' expected power is supplied for ranging.  If simulating, the measured
    ''' power is derived from the expected power and frequency with noise added.
    ''' </summary>
    ''' <param name="frequency">signal frequency, in Hz</param>
    ''' <param name="power">expected power, in dBm</param>
    ''' <returns></returns>
    Private Function MeasurePower(ByVal frequency As Double, ByVal power As Double) As Double
        Return mPowerMeter.MeasurePower(frequency, power)
    End Function

    ''' <summary>
    ''' Check all the instruments for errors.  If any, throw an exception
    ''' containing the first error found.
    ''' </summary>
    Private Sub CheckForErrors()
        mSource.CheckForErrors()
        mPowerMeter.CheckForErrors()

        Dim errorCode As Integer = 0
        Dim errorMessage As String = ""
        mVsa.Utility.ErrorQuery(errorCode, errorMessage)
        If errorCode <> 0 Then
            Throw New Exception([String].Format("VSA error: {0},{1}", errorCode, errorMessage))
        End If
    End Sub





End Class