﻿Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.Threading
Imports System.Windows.Forms
Imports System.IO

Namespace VB_FieldCalibration
    ''' <summary>
    ''' PowerMeter uses SCPI to control a variety of power meters using a VISA session.
    ''' 
    ''' If Simulated is set to true, it ignores the VISA resource and simply returns a
    ''' power without any error.
    ''' 
    ''' If Simulated is set to false, it checks the identify of the source using *IDN?
    ''' and if it is a recognized instrument, uses the SCPI for that instrument.  If
    ''' the instrument is unrecognized, a prompt will display asking which instrument
    ''' to emulate (i.e. the user selects what SCPI will be sent) -- this allows the
    ''' support for a wider range of instruments that "just happen" to recognize the
    ''' SCPI for one of the recognized instruments.
    ''' </summary>
    Public Class PowerMeter
        Inherits VisaSession
        Private Const SCPI_MODEL_UNKNOWN As Integer = 0
        Private Const SCPI_MODEL_VSERIES As Integer = 1
        Private Const SCPI_MODEL_N191X As Integer = 2

        Private Const MAXIMUM_POWER_ITERATIONS As Integer = 250
        Private Const FLATNESS_EPSILON As Double = 0.01
        ' dB
        Private Const BAD_CONNECTION_LIMIT As Double = 7.5
        ' dB
        Private ReadOnly mRandomSequence As New Random()
        ' for simulation
#Region "SCPI definitions"

        ' Definitions are updated in CheckModel()

        ' SCPI generated with: mCalAndZeroFormat
        Private mCalAndZeroFormat As String

        ' SCPI generated with: String.Format(mFrequencyFormat, mFrequencyScale*frequency);
        Private mFrequencyFormat As String
        Private mFrequencyScale As Double

        ' SCPI generated with: String.Format(mPowerFormat, mPowerScale*expectedPower);
        Private mPowerFormat As String
        Private mPowerScale As Double

        ' Local delay to insure the setup is stable...
        Private mSetupDelayMs As Integer

        ' Local delay to insure instrument has sufficient time between measurement requests
        Private mMeasureDelayUs As Double

        ' SCPI generated with:  mMeasureFormat
        Private mMeasureFormat As String

        ' SCPI generated with:  mOperationComplete
        Private mOperationComplete As String

        ' SCPI generated with:  String.Format(mMarkerFormat, mMarkerScale*frequency);
        Private mMarkerFormat As String
        Private mMarkerScale As Double

        ' SCPI generated with:  mResultFormat
        Private mResultFormat As String
        Private mResultScale As Double

#End Region

        ''' <summary>
        ''' Simulated indicates if Source is connected to real hardware or not (Auto-Property)
        ''' </summary>
        Public Property Simulated() As Boolean
            Get
                Return m_Simulated
            End Get
            Set(ByVal value As Boolean)
                m_Simulated = value
            End Set
        End Property
        Private m_Simulated As Boolean

        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="useSimulated">true to simulate source, false to use hardware</param>
        ''' <param name="resource">VISA resource - ignored if useSimulated is true</param>
        Public Sub Open(ByVal useSimulated As Boolean, ByVal resource As String)
            Try
                Simulated = useSimulated

                ' If simulated, nothing else to do
                If Simulated Then
                    Return
                End If

                ' Open VISA session
                OpenSession(resource)

                ' Determine what SCPI model we'll use -- this will also put the
                ' instrument is the correct initial state (i.e. measure CW power)
                CheckModel()
            Catch generatedExceptionName As Exception
                ' Make sure the session is closed
                CloseSession()

                ' Re-throw the exception...
                Throw
            End Try
        End Sub

        ''' <summary>
        ''' Close the VISA session
        ''' </summary>
        Public Sub Close()
            ' If simulated, nothing else to do
            If Simulated Then
                Return
            End If

            ' Close VISA session
            CloseSession()
        End Sub

        ''' <summary>
        ''' Perform a power zero/calibration on the power meter.
        ''' </summary>
        Public Sub Calibrate()
            ' If simulated or no defined SCPI, nothing else to do
            If Simulated OrElse [String].IsNullOrEmpty(mCalAndZeroFormat) Then
                Return
            End If

            ' Execute cal and *OPC?
            mSession.WriteString(mCalAndZeroFormat)

            If Not [String].IsNullOrEmpty(mOperationComplete) Then
                ' Optionally wait for completion handshake
                mSession.WriteString(mOperationComplete)
                mSession.ReadString(1000)
            End If
        End Sub

        ''' <summary>
        ''' Measure the signal power at the specified frequency.  The approximate
        ''' expected power is supplied for ranging.  If simulating, the measured
        ''' power is derived from the expected power and frequency with noise added.
        ''' </summary>
        ''' <param name="frequency">signal frequency, in Hz</param>
        ''' <param name="expectedPower">expected power, in dBm</param>
        ''' <returns></returns>
        Public Function MeasurePower(ByVal frequency As Double, ByVal expectedPower As Double) As Double
            Dim scpi As String

            If Simulated Then
                ' Simulate loss proportional to frequency and a little noise
                Return expectedPower - 0.0000000003 * frequency + 0.01 * mRandomSequence.NextDouble()
            End If

            If IsSessionOpen() Then
                ' Format and send SCPI
                If Not [String].IsNullOrEmpty(mFrequencyFormat) Then
                    ' Optionally tell the device the expected frequency
                    scpi = [String].Format(mFixedCulture, mFrequencyFormat, mFrequencyScale * frequency)
                    mSession.WriteString(scpi)
                End If

                If Not [String].IsNullOrEmpty(mPowerFormat) Then
                    ' Optionally tell the device the expected power
                    scpi = [String].Format(mFixedCulture, mPowerFormat, mPowerScale * expectedPower)
                    mSession.WriteString(scpi)
                End If

                ' Setup delay... these gives the system time to settle from the
                ' transients introduced by changing frequency/power.  The transients
                ' can be non-linear, so the simple settling algorithm below does
                ' not work correctly unless we delay past these transients. An
                ' example transient shape using a thermocouple power sensor (8485A)
                ' looks like (power vs. time):
                '
                '                                 xxxxxxxxxxxxxxxxxxxxxx
                '                         xxxxxxxx
                '   x                xxxxx
                '    xx          xxxx
                '      xxx    xxx
                '         xxxx
                '
                ' The intent of the delay is to move into the portion of the
                ' measurement that is monatonically increasing (or decreasing)
                ' which the settling algorithm below handles.
                Thread.Sleep(mSetupDelayMs)

                ' Power meters with thermocouple sensors have a non-negligible
                ' thermal mass that results in a significant settling time. To
                ' automatically determine the correct settling time, make
                ' repeated measurements until the last 3 points shows an
                ' inflection point (namely, the value is done slewing toward
                ' the actual value) OR all three values are "flat" (i.e. within
                ' FLATNESS_EPSILON=0.01 dB of each other).  The various cases
                ' for the last 3 points:
                '   values[0] < values[1] < values[2] ... still settling
                '   values[0] < values[1] > values[2] ... settled
                '   values[0] > values[1] > values[2] ... still settling
                '   values[0] > values[1] < values[2] ... settled
                '   values[0] ~ values[1] ~ values[2] ... settled (within FLATNESS_EPSILON)
                Dim tries As Integer = 0
                Dim maxTries As Integer = 5 * MAXIMUM_POWER_ITERATIONS
                ' allow for transient for first point
                Dim values As New List(Of Double)()
                Dim history As New List(Of Double)()
                values.Add(0)
                values.Add(GetNextPower(frequency, history))
                values.Add(GetNextPower(frequency, history))
                Do
                    ' Could be a length loop ... service GUI
                    Application.DoEvents()

                    ' Check if maximum # iterations exceeded
                    tries += 1
                    If tries >= maxTries Then
                        Throw New Exception([String].Format("Power measurement at {0} GHz unstable after {1} attempts. Is Power Meter warmed up?", frequency * 0.000000001, tries))
                    End If
                    maxTries = MAXIMUM_POWER_ITERATIONS
                    ' 2nd ... nth iteration limit
                    ' Remove oldest power
                    values.RemoveAt(0)
                    ' Get newest power
                    Dim value As Double = GetNextPower(frequency, history)
                    values.Add(value)

                    ' Sanity check
                    If value < expectedPower - BAD_CONNECTION_LIMIT Then
                        Throw New Exception([String].Format("Unreasonably low power ({0:0.000} dBm). Is a cable disconnected?", value))

                        ' Check for inflection point or sufficiently flat or duplicate data
                    End If
                    ' implies mMeasureDelayUs is too small
                    ' implies mMeasureDelayUs is too small
                Loop While (values(0) < values(1) AndAlso values(1) < values(2)) OrElse (values(0) > values(1) AndAlso values(1) > values(2)) OrElse Math.Abs(values(0) - values(1)) > FLATNESS_EPSILON OrElse Math.Abs(values(1) - values(2)) > FLATNESS_EPSILON OrElse Math.Abs(values(0) - values(2)) > FLATNESS_EPSILON OrElse values(0) = values(1) OrElse values(1) = values(2)

                ' And the answer is...
                Dim result As Double = (values(0) + values(1) + values(2)) / 3

                ' For debugging... compare the result with dynamic iterations to the result
                ' waiting for MAXIMUM_POWER_ITERATIONS
                'double debugValue = result;
                'int debugTries = tries;
                'do
                '{
                '    Application.DoEvents();
                '    debugValue = GetNextPower(frequency, ref history);
                '    debugTries += 1;
                '} while (debugTries < MAXIMUM_POWER_ITERATIONS);
                'Console.WriteLine( String.Format("Measured {1:0.000} dBm using {0} iterations. {2:0.000} dBm at MAXIMUM_POWER_ITERATIONS", tries, result, debugValue));
                'if (Math.Abs(debugValue - result) > 0.01)
                '{
                '    Console.WriteLine( "Persisting history" );
                '    Persist(ref history);
                '}

                Return result
            End If

            Throw New Exception("No connection to PowerMeter")
        End Function

        ''' <summary>
        ''' For debugging -- write a list of values to a file.  Format
        ''' is CSV ... appropriate for importing into Excel
        ''' </summary>
        ''' <param name="history"></param>
        Private Sub Persist(ByRef history As List(Of Double))
            Dim writer As TextWriter = New StreamWriter("history.txt", True)
            For Each value In history
                writer.Write([String].Format("{0:0.000},", value))
            Next
            writer.WriteLine("")
            writer.Close()
        End Sub

        ''' <summary>
        ''' Get a power value from the power meter.
        ''' </summary>
        ''' <param name="frequency"></param>
        ''' <returns></returns>
        Private Function GetNextPower(ByVal frequency As Double, ByRef history As List(Of Double)) As Double
            If Not [String].IsNullOrEmpty(mMeasureFormat) Then
                ' If in single/triggered mode, invoke the measurement
                mSession.WriteString(mMeasureFormat)
            End If

            ' for free run measurements, need to make sure the instrument has
            ' had time to make another measurement
            SpinWait(mMeasureDelayUs)

            If Not [String].IsNullOrEmpty(mOperationComplete) Then
                ' Optionally wait for completion handshake
                mSession.WriteString(mOperationComplete)
                mSession.ReadString(1000)
            End If

            If Not [String].IsNullOrEmpty(mMarkerFormat) Then
                ' Optionally tell the device the expected measurement frequency
                Dim scpi As String = [String].Format(mFixedCulture, mMarkerFormat, mMarkerScale * frequency)
                mSession.WriteString(scpi)
            End If

            ' Finally, request and read the result
            mSession.WriteString(mResultFormat)
            Dim result As String = mSession.ReadString(1000)

            Dim value As Double = mResultScale * [Double].Parse(result, mFixedCulture)
            If history IsNot Nothing Then
                history.Add(value)
            End If
            Return value
        End Function

        ''' <summary>
        ''' Use the high performance counters (via Stopwatch) to implement
        ''' a spin-loop wait.  This method should only be used for delays of
        ''' a few milliseconds or less.
        ''' </summary>
        ''' <param name="delayUs"></param>
        Private Sub SpinWait(ByVal delayUs As Double)
            If delayUs > 0 Then
                Dim minTicks As Double = Stopwatch.Frequency * delayUs / 1000000.0
                Dim timer As New Stopwatch()
                timer.Start()
                While timer.ElapsedTicks < minTicks
                    Thread.Sleep(0)
                End While
                timer.[Stop]()
            End If

        End Sub

        ''' <summary>
        ''' Check the instrument for errors.  If there are any errors,
        ''' throw the first error as an exception.
        ''' </summary>
        Public Sub CheckForErrors()
            If Simulated Then
                Return
            End If

            mSession.WriteString(":SYST:ERR?" & vbLf)
            Dim response As String = mSession.ReadString(1000)
            Dim separators As Char() = {","c}
            Dim fields As String() = response.Split(separators)
            If Int32.Parse(fields(0)) <> 0 Then
                Throw New Exception("Power Meter error: " & response)
            End If
        End Sub

        ''' <summary>
        ''' Use the *IDN? query to determine what the connected instrument model
        ''' is.  If it is one of the supported instruments, use it to define the
        ''' SCPI commands.  If it is not, ask the user to select one of the supported
        ''' instruments as a "SCPI model" to use and define the SCPI commands based
        ''' on the selected instrument model.
        ''' </summary>
        Private Sub CheckModel()
            ' Ask instrument for identity
            mSession.WriteString("*IDN?" & vbLf)
            Dim idn As String = mSession.ReadString(1000)

            ' Parse *IDN? response
            Dim separators As Char() = {","c}
            Dim fields As String() = idn.Split(separators)
            If fields.Length <> 4 Then
                Throw New Exception("Unexpected response to *IDN? - " & idn)
            End If
            Dim manufacturer As String = fields(0)
            Dim model As String = fields(1)

            ' Supported?
            Dim scpiModel As Integer = SCPI_MODEL_UNKNOWN
            If manufacturer.ToUpper().Contains("AGILENT") Then
                ' For development purposes - recognize V-Series
                If model = "MODEL 2810" OrElse model = "MODEL 2820" OrElse model = "MODEL 2820A" OrElse model = "V2820A" Then
                    scpiModel = SCPI_MODEL_VSERIES
                ElseIf model = "N9313A" OrElse model = "N9314A" Then
                    scpiModel = SCPI_MODEL_N191X
                End If
            ElseIf manufacturer.ToUpper().Contains("HEWLETT-PACKARD") Then
                If model = "E4418B" Then
                    scpiModel = SCPI_MODEL_N191X
                End If
            End If

            If scpiModel = SCPI_MODEL_UNKNOWN Then
                ' Ask user if one of the supported instruments is compatible
                ' with the connected instrument...
                Dim connected As String = [String].Format("{0} {1}", manufacturer, model)
                Dim compatibleList As String() = {"N9313A", "N9314A"}
                Dim dialog As New ChooseModel(connected, compatibleList)
                If dialog.ShowDialog() = DialogResult.OK Then
                    ' Convert result to enumeration
                    Dim compatible As String = dialog.CompatibleInstrument
                    If compatible = "N9313A" OrElse compatible = "N9314A" Then
                        scpiModel = SCPI_MODEL_N191X
                    End If
                End If
            End If

            ' Setup instruement to measure CW
            Select Case scpiModel
                Case SCPI_MODEL_VSERIES
                    ' Define SCPI
                    mCalAndZeroFormat = Nothing
                    mFrequencyFormat = ":FREQ:CENT {0:0.0########} GHz" & vbLf
                    mFrequencyScale = 0.000000001
                    mPowerFormat = ":EPOW {0:0.0}" & vbLf
                    mPowerScale = 1.0
                    mSetupDelayMs = 50
                    mMeasureFormat = ":INIT:IMM" & vbLf
                    mMeasureDelayUs = 0
                    mOperationComplete = "*OPC?" & vbLf
                    mMarkerFormat = ":CALC:MARK1:X {0:0.0########} GHz" & vbLf
                    mMarkerScale = 0.000000001
                    mResultFormat = ":CALC:MARK1:Y?" & vbLf
                    mResultScale = 1.0

                    ' Initialize/setup up instrument for CW power measurement
                    mSession.WriteString("*RST;*OPC?" & vbLf)
                    mSession.ReadString(1000)
                    mSession.WriteString(":INIT:CONT 0" & vbLf)
                    mSession.WriteString(":FREQ:SPAN 1 MHz" & vbLf)
                    mSession.WriteString(":CALC:MARK1:STATE 1" & vbLf)
                    mSession.WriteString("*CLS;*OPC?" & vbLf)
                    mSession.ReadString(1000)
                    Exit Select

                Case SCPI_MODEL_N191X
                    ' Define SCPI
                    mCalAndZeroFormat = "CAL" & vbLf
                    mFrequencyFormat = "FREQ {0:0.0########} GHz" & vbLf
                    mFrequencyScale = 0.000000001
                    mPowerFormat = Nothing
                    mPowerScale = 1.0
                    ' mSetupDelayMs is empirically determined to make sure the transients from
                    ' changing frequency/power have passed -- 500 ms is too small.
                    mSetupDelayMs = 750
                    mMeasureFormat = Nothing
                    ' "INIT\n"; // only use INIT if :INIT:CONT OFF
                    mMeasureDelayUs = 5000
                    mOperationComplete = Nothing
                    '"*OPC?\n"; // only use *OPC? if :INIT:CONT OFF
                    mMarkerFormat = Nothing
                    mMarkerScale = 0.000000001
                    mResultFormat = "FETC?" & vbLf
                    mResultScale = 1.0

                    ' Initialize/setup up instrument for CW power measurement
                    mSession.WriteString("SYST:PRES" & vbLf)
                    ' TODO: when to run a cal?
                    'mSession.WriteString("CAL\n");
                    'mSession.ReadString(1000);
                    mSession.WriteString(":INIT:CONT ON" & vbLf)
                    mSession.WriteString("*CLS;*OPC?" & vbLf)
                    mSession.ReadString(1000)
                    Exit Select
                Case Else

                    Throw New Exception("Unsupported instrument: " & idn)
            End Select
        End Sub
    End Class
End Namespace
