﻿Imports System.Windows.Forms

Namespace VB_FieldCalibration
    ''' <summary>
    ''' Source uses SCPI to control a variety of signal sources using a VISA session.
    ''' 
    ''' If Simulated is set to true, it ignores the VISA resource and simply "swallows"
    ''' requests to set frequency and power without any error.
    ''' 
    ''' If Simulated is set to false, it checks the identify of the source using *IDN?
    ''' and if it is a recognized instrument, uses the SCPI for that instrument.  If
    ''' the instrument is unrecognized, a prompt will display asking which instrument
    ''' to emulate (i.e. the user selects what SCPI will be sent) -- this allows the
    ''' support for a wider range of instruments that "just happen" to recognize the
    ''' SCPI for one of the recognized instruments.
    ''' </summary>
    Public Class Source
        Inherits VisaSession
        ''' <summary>
        ''' Cached state of the signal output enable
        ''' </summary>
        Private mOutputEnabled As Boolean

        Private Const SCPI_MODEL_UNKNOWN As Integer = 0
        Private Const SCPI_MODEL_PSG As Integer = 1
        ' Agilent E8257D, E8267D, ...
#Region "SCPI definitions"

        ' Definitions are updated in CheckModel()

        ' SCPI generated with:  String.Format(mOutputEnableFormat, enabled ? mOutputEnableStateOn : mOutputEnableStateOff);
        Private mOutputEnableFormat As String
        Private mOutputEnableStateOn As String
        Private mOutputEnableStateOff As String

        ' SCPI generated with:  String.Format(mFrequencyFormat, mFrequencyScale * frequency);
        Private mFrequencyFormat As String
        Private mFrequencyScale As Double

        ' SCPI generated with:  String.Format(mPowerFormat, mPowerScale * power);
        Private mPowerFormat As String
        Private mPowerScale As Double

        ' SCPI generated with:   mSettled  (expects a query response)
        Private mSettled As String

#End Region

        ''' <summary>
        ''' Simulated indicates if Source is connected to real hardware or not (Auto-Property)
        ''' </summary>
        Public Property Simulated() As Boolean
            Get
                Return m_Simulated
            End Get
            Set(ByVal value As Boolean)
                m_Simulated = value
            End Set
        End Property
        Private m_Simulated As Boolean

        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="useSimulated">true to simulate source, false to use hardware</param>
        ''' <param name="resource">VISA resource - ignored if useSimulated is true</param>
        Public Sub Open(ByVal useSimulated As Boolean, ByVal resource As String)
            Try
                Simulated = useSimulated

                ' If simulated, nothing else to do
                If Simulated Then
                    Return
                End If

                ' Open VISA session
                OpenSession(resource)

                ' Determine what SCPI model we'll use -- this will also put the
                ' instrument is the correct initial state (i.e. CW signal generation)
                CheckModel()
            Catch generatedExceptionName As Exception
                ' Make sure the session is closed
                CloseSession()

                ' Re-throw the exception...
                Throw
            End Try
        End Sub

        ''' <summary>
        ''' Turn off the signal output and close the VISA session
        ''' </summary>
        Public Sub Close()
            ' If simulated, nothing else to do
            If Simulated Then
                Return
            End If

            ' Make sure the output is off
            SetOutput(False)

            ' Close VISA session
            CloseSession()
        End Sub

        ''' <summary>
        ''' Set the source to the desired frequency and power then wait
        ''' until the source is settled.
        ''' </summary>
        ''' <param name="frequency"></param>
        ''' <param name="power"></param>
        Public Sub Configure(ByVal frequency As Double, ByVal power As Double)
            SetFrequency(frequency)
            SetPower(power)
            SetOutput(True)
            WaitForSettled()
        End Sub

        ''' <summary>
        ''' Enable or disable the signal output.
        ''' </summary>
        ''' <param name="enabled"></param>
        Public Sub SetOutput(ByVal enabled As Boolean)
            ' If simulated, nothing else to do
            If Simulated Then
                Return
            End If

            If IsSessionOpen() Then
                ' If true, only re-send SCPI if cached/previous value was false
                If enabled AndAlso mOutputEnabled Then
                    Return
                End If

                ' Cache value
                mOutputEnabled = enabled

                ' Format and send SCPI
                Dim scpi As String = [String].Format(mOutputEnableFormat, If(enabled, mOutputEnableStateOn, mOutputEnableStateOff))
                mSession.WriteString(scpi)
            End If
        End Sub

        ''' <summary>
        ''' Set the frequency of the signal
        ''' </summary>
        ''' <param name="frequency">frequency, in hertz</param>
        Public Sub SetFrequency(ByVal frequency As Double)
            ' If simulated, nothing else to do
            If Simulated Then
                Return
            End If

            If IsSessionOpen() Then
                ' Format and send SCPI
                Dim scpi As String = [String].Format(mFixedCulture, mFrequencyFormat, mFrequencyScale * frequency)
                mSession.WriteString(scpi)
            End If
        End Sub

        ''' <summary>
        ''' Set the power of the signal
        ''' </summary>
        ''' <param name="power">power, in dBm</param>
        Public Sub SetPower(ByVal power As Double)
            ' If simulated, nothing else to do
            If Simulated Then
                Return
            End If

            If IsSessionOpen() Then
                ' Format and send SCPI
                Dim scpi As String = [String].Format(mFixedCulture, mPowerFormat, mPowerScale * power)
                mSession.WriteString(scpi)
            End If
        End Sub

        ''' <summary>
        ''' Wait until the source output is stable.
        ''' </summary>
        Public Sub WaitForSettled()
            If Simulated Then
                ' If simulated, simply delay a bit to simulate real hardware
                System.Threading.Thread.Sleep(10)
                Return
            End If

            If IsSessionOpen() Then
                ' Send SCPI -- this should generate a response for handshaking
                mSession.WriteString(mSettled)
                mSession.ReadString(1000)
            End If
        End Sub

        ''' <summary>
        ''' Check the instrument for errors.  If there are any errors,
        ''' throw the first error as an exception.
        ''' </summary>
        Public Sub CheckForErrors()
            If Simulated Then
                Return
            End If

            mSession.WriteString(":SYST:ERR?" & vbLf)
            Dim response As String = mSession.ReadString(1000)
            Dim separators As Char() = {","c}
            Dim fields As String() = response.Split(separators)
            If Int32.Parse(fields(0)) <> 0 Then
                Throw New Exception("Source error: " & response)
            End If
        End Sub

        ''' <summary>
        ''' Use the *IDN? query to determine what the connected instrument model
        ''' is.  If it is one of the supported instruments, use it to define the
        ''' SCPI commands.  If it is not, ask the user to select one of the supported
        ''' instruments as a "SCPI model" to use and define the SCPI commands based
        ''' on the selected instrument model.
        ''' </summary>
        Private Sub CheckModel()
            ' Ask instrument for identity
            mSession.WriteString("*IDN?" & vbLf)
            Dim idn As String = mSession.ReadString(1000)

            ' Parse *IDN? response
            Dim separators As Char() = {","c}
            Dim fields As String() = idn.Split(separators)
            If fields.Length <> 4 Then
                Throw New Exception("Unexpected response to *IDN? - " & idn)
            End If
            Dim manufacturer As String = fields(0).Trim()
            Dim model As String = fields(1).Trim()

            ' Supported?
            Dim scpiModel As Integer = SCPI_MODEL_UNKNOWN
            If manufacturer.ToUpper().Contains("AGILENT") Then
                If model = "E8257D" OrElse model = "E8267D" Then
                    scpiModel = SCPI_MODEL_PSG

                End If
            End If

            If scpiModel = SCPI_MODEL_UNKNOWN Then
                ' Ask user if one of the supported instruments is compatible
                ' with the connected instrument...
                Dim connected As String = [String].Format("{0} {1}", manufacturer, model)
                Dim compatibleList As String() = {"E8257D", "E8267D"}
                Dim dialog As New ChooseModel(connected, compatibleList)
                If dialog.ShowDialog() = DialogResult.OK Then
                    ' Convert result to enumeration
                    Dim compatible As String = dialog.CompatibleInstrument
                    If compatible = "E8257D" OrElse compatible = "E8267D" Then
                        scpiModel = SCPI_MODEL_PSG
                    End If
                End If
            End If

            ' Setup instruement to generate CW
            Select Case scpiModel
                Case SCPI_MODEL_PSG
                    ' Define SCPI
                    mOutputEnableFormat = ":OUTP {0}" & vbLf
                    mOutputEnableStateOn = "1"
                    mOutputEnableStateOff = "0"
                    mFrequencyFormat = ":FREQ {0:0.0########} GHz" & vbLf
                    mFrequencyScale = 0.000000001
                    mPowerFormat = ":POW {0:0.0#}" & vbLf
                    mPowerScale = 1.0
                    mSettled = "*OPC?" & vbLf

                    ' Initialize/setup up instrument for CW signal generation
                    mSession.WriteString("*RST;*CLS;*OPC?" & vbLf)
                    mSession.ReadString(1000)
                    Exit Select
                Case Else

                    Throw New Exception("Unsupported instrument: " & idn)
            End Select
        End Sub
    End Class
End Namespace
