﻿Imports AgM9392Lib

Module Program

	Private vsa As AgM9392
	Private preselector As Boolean
	Private rfDc As Boolean
	Private uWaveDc As Boolean
	Private configFile As String
	Private simulated As Boolean

	Sub Main()

		vsa = New AgM9392

        Dim args() As String

        args = Split(Command$)

        '   * You can change the code below to hard-code your resource descriptor and simulation mode, or you can pass the resource
        '   * descriptor in the command line when you run the example program to use that hardware in non-simulation mode.
        '   *
        '   * The resource descriptor is the same value as the “Selected Instrument” field of the SFP’s (Soft Front Panel)
        '   * connection dialog.   This can be a comma or semicolon separated list of module addresses such as
        '   *     PXI10::15::0::INSTR;PXI10::16::0::INSTR;PXI10::17::0::INSTR;PXI10::18::0::INSTR
        '   * or the name of a saved “Instrument Connection” created by the SFP such as
        '   *     PreselectedVSA
        '   * Refer to the SFP Help topic “Connect to Instruments and Modules” for more details.

        ' Pass in a command line argument as the resource descriptor, if none, will default
        Dim resource As String = "PXI20::13::INSTR"
        Dim options As String = "QueryInstrStatus=true, Simulate=true, DriverSetup= Model=, Trace=false"
        If ((args.Length > 0) And (args(0).Length > 0)) Then
            resource = args(0)
            options = "QueryInstrStatus=true, Simulate=false, DriverSetup= Model=, Trace=false"
        End If

		' Make a connection to the M9392 VSA interface
        vsa.Initialize(resource, True, True, options)

		Try
			'Determine what hardware we have available to us to work with.
			DetermineHardware()

			'If you don't have a trigger line hooked up to the M9202A Digitizer, then change this to Free Run.
			ConfigureTriggering(AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal)

			DemonstrateAttenuationHint()

			DemonstratePathPreferenceHints()

		Catch ex As Exception
			Console.WriteLine("Caught an exception during execution\n{0}\nat:\n{1}", ex.Message, ex.StackTrace)
		Finally
			vsa.Close()
		End Try

	End Sub

	' <summary>
	' The RF and uWave paths through the M9392 VSA driver overlap between 2.75 and 2.90 GHz due to capabilities
	' of each of the downconverters.  The M9392 VSA driver will determine the best path to take based on frequency
	' and bandwidth at the time of a Measurements.Initiate() command.  Between 2.75 and 2.90 GHz both paths
	' forming "receivers" are valid and without any hint, the M9392 will bias towards using the uWave receiver.
	' Because of the characteristics of the two downconverters it may be desirable during execution of Initiate()
	' to "prefer" that the M9392 pick a different path.  There are two hints "Prefer RF Downconverter" and 
	' "Prefer UWave Downconverter" that can be used to influence the selection of the path in overlapping
	' bands.
	' </summary>
	Sub DemonstratePathPreferenceHints()
		'This only makes sense if the system as both a RF and uWave downconverter
		If Not rfDc Or Not uWaveDc Then
			Console.WriteLine("Skipping DemonstratePathPreferenceHints because the M9392 VSA has only a rf or uWave Downconverter")
			Return
		End If

		vsa.Status.Clear()
		vsa.Acquisition.Hints.Clear()

		' Make a measurement at 2.8 GHZ @ -10 dBm, 40 MHz BW, with a 1 us acquisition time
		vsa.Acquisition.PreselectorEnabled = False
		vsa.Acquisition.Frequency = 2800000000.0
		vsa.Acquisition.Power = -10
		vsa.Acquisition.Bandwidth = 40000000.0
		vsa.Acquisition.Time = 0.0001

		' First time, force the VSA to pick the RF path

		vsa.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypePreferRfDownConverter, 0) ' 2nd Parameter is meaningless...

		vsa.Measurements.Initiate()
		vsa.Measurements.WaitForInitiateComplete(1000)
		vsa.Measurements.Arm()

		Dim power As Double = vsa.Measurements.PeakPower

		'Second time, force the VSA to pick the uWave path.
		vsa.Acquisition.Hints.Clear()
		vsa.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypePreferUWaveDownConverter, 0) '2nd Parameter is meaningless...

		vsa.Measurements.Initiate()
		vsa.Measurements.WaitForInitiateComplete(1000)
		vsa.Measurements.Arm()

		Dim powerAfter As Double = vsa.Measurements.PeakPower
		Dim deltaPower As Double = power - powerAfter

		Console.WriteLine("RF Path and uWave Path yielded difference in peak power is: {0}", deltaPower)
	End Sub
	' <summary>
	' Demonstrate the use of the "Force Front End Attenuation" hint.
	' Under normal usage the M9392 VSA driver will take the input power set by the client application
	' and determine the correct attenuation setting for the front-end preselector, however, there may be
	' circumstances that the client would want to force the front end attenuator to some other setting.
	' The M9360A preselector's step attenuator works in increments of 10 dB, with valid values of [0 dB, 70 dB].
	' </summary>
	Sub DemonstrateAttenuationHint()
		'if we don't have a preselector, this example is meaningless
		If Not preselector Then
			Console.WriteLine("Skipping DemonstrateAttenuationHint because the M9392 VSA doesn't have a M9360A Preselector")
			Return
		End If

		'Show the front end attenuation hint using the RF Downconverter if there is one.
		If rfDc Then
			vsa.Status.Clear()
			vsa.Acquisition.Hints.Clear()

			'Make a measurement at 2.4 GHZ @ -10 dBm, 40 MHz BW, with a 1 us acquisition time
			vsa.Acquisition.PreselectorEnabled = False
			vsa.Acquisition.Frequency = 2400000000.0
			vsa.Acquisition.Power = -10
			vsa.Acquisition.Bandwidth = 40000000.0
			vsa.Acquisition.Time = 0.0001

			vsa.Measurements.Initiate()
			vsa.Measurements.WaitForInitiateComplete(1000)
			vsa.Measurements.Arm()

			Dim power As Double = vsa.Measurements.RmsPower

			Dim preselectorAtten As Single = 0
			Dim dcAtten1 As Single = 0
			Dim dcAtten2 As Single = 0

			'Get what the system set the internal attenuators to
			vsa.Measurements.GetAttenuatorSettings(preselectorAtten, dcAtten1, dcAtten2)

			'Now force the front-end attenuation to 10 dB above what it automatically picked
			Dim newAtten As Double = preselectorAtten + 10
			vsa.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeForceFrontEndAttenuationTo, newAtten)

			vsa.Measurements.Initiate()
			vsa.Measurements.WaitForInitiateComplete(1000)
			vsa.Measurements.Arm()

			If (vsa.Measurements.DidAcquisitionOverload) Then
				Console.WriteLine("The VSA is overloaded.")
			End If

			'Get what the system set the internal attenuators to (again)
			vsa.Measurements.GetAttenuatorSettings(preselectorAtten, dcAtten1, dcAtten2)

			' If the preselector now doesn't match what we forced it to, then something bad happened.
			If newAtten <> preselectorAtten Then
				Console.WriteLine("Attention, the hint apparently wasn't used!")
			End If

			Dim powerAfter As Double = vsa.Measurements.RmsPower
			Dim deltaPower As Double = power - powerAfter

			Console.WriteLine("Increased Front End Attenation to RF Path yielded difference in RMS power is: {0}", deltaPower)

		End If

		' Show the front end attenuation hint using the RF Downconverter if there is one.
		If uWaveDc Then
			vsa.Status.Clear()
			vsa.Acquisition.Hints.Clear()

			' Make a measurement at 5.0 GHZ @ -10 dBm, 40 MHz BW, with a 1 us acquisition time
			vsa.Acquisition.PreselectorEnabled = False
			vsa.Acquisition.Frequency = 5000000000.0
			vsa.Acquisition.Power = -10
			vsa.Acquisition.Bandwidth = 40000000.0
			vsa.Acquisition.Time = 0.0001

			vsa.Measurements.Initiate()
			vsa.Measurements.WaitForInitiateComplete(1000)
			vsa.Measurements.Arm()

			Dim power As Double = vsa.Measurements.RmsPower

			Dim preselectorAtten As Single = 0
			Dim dcAtten1 As Single = 0
			Dim dcAtten2 As Single = 0

			' Get what the system set the internal attenuators to
			vsa.Measurements.GetAttenuatorSettings(preselectorAtten, dcAtten1, dcAtten2)

			'Now force the front-end attenuation to 10 dB above what it automatically picked
			Dim newAtten As Double = preselectorAtten + 10
			vsa.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeForceFrontEndAttenuationTo, newAtten)

			vsa.Measurements.Initiate()
			vsa.Measurements.WaitForInitiateComplete(1000)
			vsa.Measurements.Arm()

			If vsa.Measurements.DidAcquisitionOverload Then
				Console.WriteLine("The VSA is overloaded.")
			End If

			'Get what the system set the internal attenuators to (again)
			vsa.Measurements.GetAttenuatorSettings(preselectorAtten, dcAtten1, dcAtten2)

			'If the preselector now doesn't match what we forced it to, then something bad happened.
			If newAtten <> preselectorAtten Then
				Console.WriteLine("Attention, the hint apparently wasn't used!")
			End If

			Dim powerAfter As Double = vsa.Measurements.RmsPower

			Dim deltaPower As Double = power - powerAfter

			Console.WriteLine("Increased Front End Attenation to uWave Path yielded Difference in RMS power is: {0}", deltaPower)
		End If

	End Sub

	Sub DetermineHardware()

		For index As Int32 = 0 To vsa.System.ModuleCount - 1

			Dim manufacturer As String = String.Empty
			Dim model As String = String.Empty
			Dim fwRev As String = String.Empty
			Dim serial As String = String.Empty

			vsa.System.GetModuleIdentity(index, manufacturer, model, fwRev, serial)

			Select Case model
				Case "M9360A", "PXI-1410"
					preselector = True
				Case "M9351A"
					rfDc = True
				Case "M9361A"
					uWaveDc = True
			End Select
		Next

	End Sub

	Sub ConfigureTriggering(ByVal trigger As AgM9392TriggerSourceEnum)
		Select Case trigger
			Case AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal
				vsa.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal
				vsa.Trigger.Delay = 0
				vsa.Trigger.External.Configure(1, AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive)
			Case AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate
				vsa.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate
			Case AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideo
				Console.WriteLine("Video triggering isn't configurable in this example")
		End Select
	End Sub

End Module
